from PyQt5.QtWidgets import QWidget,QHBoxLayout,QTableWidget,QPushButton,QApplication,QVBoxLayout,QTableWidgetItem,QCheckBox,QAbstractItemView,QHeaderView,QLabel,QFrame,QLineEdit,QComboBox,QGridLayout,QGroupBox,QMainWindow,QRadioButton, QToolBar, QMessageBox
from PyQt5 import QtWidgets,QtCore
from PyQt5.QtCore import pyqtSignal, QPoint, Qt, QSize, QRectF, QPointF, QTimer
from PyQt5.QtGui import  QFont,QColor, QPixmapCache, QIcon, QPen
#from faker import Factory
import random
import sys
import pyqtgraph as pg
from pyqtgraph import GraphicsLayoutWidget, mkPen, mkBrush
from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph.graphicsItems.ViewBox import ViewBox
from collections import defaultdict
from pyqtgraph import Point, GraphicsObject
from numpy import errstate,isneginf,array
from .QWidget_widget import label_widget, button_widget
from .mrc_mnc_backend import mrc_mnc
import math
from .math_func import math_func
import datetime
import numpy as np
class Table_widget(QTableWidget):
    def __init__(self):
        super().__init__()
        #self = QTableWidget(self)
        #self.setFixedSize(350,120)
        self.setSelectionBehavior(1)
        font = QFont('Times New Roman', 12)
        font.setBold(True)
        self.setColumnCount(5)
        self.setRowCount(2)
        self.setItem(0,0,QTableWidgetItem(None))
        self.setItem(0,1,QTableWidgetItem(None))
        self.setItem(0,2,QTableWidgetItem(str(0.1)))
        self.setItem(0,3,QTableWidgetItem(str(0.1)))
        self.setItem(0,4,QTableWidgetItem("绝对位置"))
        self.setItem(1,0,QTableWidgetItem(str(-1.0)))
        self.setItem(1,1,QTableWidgetItem(str(1.0)))
        self.setItem(1,2,QTableWidgetItem(str(0.1)))
        self.setItem(1,3,QTableWidgetItem(str(0.1)))
        self.setItem(1,4,QTableWidgetItem("相对位置"))
        self.headers = ['Start','End','Step','Time(s)','']
        self.setHorizontalHeaderLabels(self.headers)
        self.horizontalHeader().setFont(font)  # 设置表头字体
        self.horizontalHeader().setStyleSheet('QHeaderView::section{background:gray}')
        self.horizontalHeader().setFixedHeight(30)
        self.setFixedHeight(90)
        #self.addItem([])
        self.horizontalHeader().setDefaultAlignment(Qt.AlignLeft | Qt.AlignVCenter)
        #self.verticalHeader().setDefaultAlignment(Qt.AlignLeft)
        self.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        #self.verticalHeader().setSectionResizeMode(QHeaderView.Fixed)
        self.verticalHeader().setSectionResizeMode(QHeaderView.Stretch)

class M_widget(QWidget):
    def __init__(self,index,value_now,mrc,mnc):
        super().__init__()
        self.mrc_mnc = mrc_mnc(mrc,mnc)
        self.motor_list_read = self.mrc_mnc.get_M_list()
        self.math_func = math_func()
        self.motor_list = self.math_func.read_M_list(self.motor_list_read)
        self.table_motor = Table_widget()
        self.motor_pv_combox = QComboBox(self)
        self.motor_pv_combox.setFixedSize(100, 40)
        #self.motor_list = M_name
        self.motor_label = label_widget("设备")
        self.motor_combox = QComboBox(self)
        self.motor_combox.setFixedSize(100, 40)
        self.motor_combox.addItems(self.motor_list.keys())
        ele_motor = list(self.motor_list.keys())
        ele_motor_pv = self.motor_list[ele_motor[0]]
        self.motor_pv_combox.addItem(ele_motor_pv[0])
        self.motor_name = label_widget("Motor_"+str(index))
        self.motor_now = label_widget("current")
        self.motor_now_lineedit = QLineEdit()
        self.motor_now_lineedit.setFixedSize(170, 40)
        self.motor_target = label_widget("target")
        self.motor_target_lineedit = QLineEdit()
        self.motor_target_lineedit.setFixedSize(100, 40)

        self.vbox0 = QVBoxLayout()
        self.vbox0.addWidget(self.motor_label)
        self.vbox0.addWidget(self.motor_combox)
        self.vbox1 = QVBoxLayout()
        self.vbox1.addWidget(self.motor_name)
        self.vbox1.addWidget(self.motor_pv_combox)
         
        self.vbox22 = QVBoxLayout()
        self.vbox22.addWidget(self.motor_target)
        self.vbox22.addWidget(self.motor_target_lineedit)
        self.vbox2 = QVBoxLayout()
        self.vbox2.addWidget(self.motor_now)
        self.vbox2.addWidget(self.motor_now_lineedit)
        self.btn_M_move = button_widget('Move')
        self.btn_M_move.setFixedSize(120, 40)
        self.btn_M_stop = button_widget('Stop')
        self.btn_M_stop.setFixedSize(120, 40)
        self.vbox23 = QVBoxLayout()
        self.vbox23.addWidget(self.btn_M_move)
        self.vbox23.addWidget(self.btn_M_stop)
        self.hbox = QHBoxLayout()
        self.hbox.addLayout(self.vbox0)
        self.hbox.addLayout(self.vbox1)
        self.hbox.addLayout(self.vbox2)
        self.hbox.addLayout(self.vbox22)
        self.hbox.addLayout(self.vbox23)
        self.vbox3 = QVBoxLayout()
        self.vbox3.addLayout(self.hbox)
        self.vbox3.addWidget(self.table_motor)
        self.vbox3.addItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding))
        groupbox1 = QGroupBox("")
        groupbox1.setLayout(self.vbox3)
        self.hhbox = QHBoxLayout()
        self.hhbox.addWidget(groupbox1)
        self.setLayout(self.hhbox)
        self.btn_M_move.clicked.connect(self.start_move)
        self.btn_M_stop.clicked.connect(self.stop_move)
        self.timer =  QTimer()
        self.starttime()
        self.init_item()
        self.sig_change = 1
        self.table_motor.cellChanged.connect(self.cellchange)
    def get_M_name(self):
        M_name_name = self.motor_combox.currentText()
        M_name_pv = self.motor_pv_combox.currentText()
        M_name = M_name_name+'_'+M_name_pv
        return(M_name)
    def start_move(self):
        M_pos = self.motor_target_lineedit.text()
        #M_name = self.motor_pv_combox.currentText()
        M_name = self.get_M_name()
        #M_name_name = self.motor_combox.currentText()
        #M_name_pv = self.motor_pv_combox.currentText()
        #M_name = M_name_name+'_'+M_name_pv
        pos_high,pos_low = self.mrc_mnc.limit_M_pos(M_name)
        print(pos_high,pos_low)
        if M_pos == "":
           self.messagebox = QMessageBox.critical(self,'提示','please input target',QMessageBox.Yes|QMessageBox.Yes)
        elif pos_low > float(M_pos) or pos_high < float(M_pos):
            self.messagebox = QMessageBox.critical(self,'提示','please check  target  in range ['+str(pos_low)+','+str(pos_high)+']',QMessageBox.Yes|QMessageBox.Yes)
        else:
        #D_loop = self.loop_lineedit.text()
           #self.btn_M_move.setEnabled(False)
           #self.btn_M_stop.setEnabled(True)
           #self.mrc_mnc.mrc.do_cmd("%go "+ M_name +".set(%f).wait()\n" % float(M_pos)).subscribe(self.cb)
           self.mrc_mnc.set_M(M_name, M_pos)#.subscribe(self.cb)
    def cb(self,rep):
        #self.btn_M_move.setEnabled(True)
        #self.btn_M_stop.setEnabled(False)
        self.motor_target_lineedit.clear()
    def stop_move(self):
        #M_name = self.motor_pv_combox.currentText()
        M_name = self.get_M_name()
        self.mrc_mnc.stop_M(M_name)
        #self.btn_M_move.setEnabled(True)
        #self.btn_M_stop.setEnabled(False)
    def update_pos_now(self):
        #M_name_name = self.motor_combox.currentText()
        #M_name_pv = self.motor_pv_combox.currentText()
        #M_name = M_name_name+'_'+M_name_pv
        M_name = self.get_M_name()
        #print(M_name)
        self.pos_now,self.pos_unit = self.mrc_mnc.get_M_pos(M_name)
        #print(self.pos_now,self.pos_unit)
        self.motor_now_lineedit.setText(str('%.4f'%self.pos_now) + self.pos_unit)
    def init_item(self):
        M_name = self.get_M_name()
        self.pos_now,self.pos_unit = self.mrc_mnc.get_M_pos(M_name)
        for i in range(2):
           item = self.table_motor.item(1,i)
           txt = item.text()
           text_input =  float(txt)+ float(self.pos_now)
           self.table_motor.setItem(0,i,QTableWidgetItem(str('%.2f'%text_input))) 
    def scan_para(self):
        M_start = self.table_motor.item(0,0).text()
        M_end = self.table_motor.item(0,1).text()
        M_step = self.table_motor.item(0,2).text()
        D_exp = self.table_motor.item(0,2).text()
        print(M_start,M_end,M_step)
        M_num = int(np.ceil((float(M_end)-float(M_start))/float(M_step))) + 1
        print(M_start,M_end,M_num)
        return(M_start,M_end,M_num,D_exp)
    def starttime(self):
        self.timer.start(200)
        self.timer.timeout.connect(self.update_pos_now)
    def cellchange(self,row,col):
        if self.sig_change % 2 : 
           self.item_change(row,col)
    def item_change(self,row,col):
        #print(row,col)
        self.sig_change = False
        item = self.table_motor.item(row,col)
        txt = item.text()
        #print(row,col)
        #print(txt)
        if (col == 0 or col == 1):
            if (row % 2 == 0):
              #self.energy_range_check(float(txt)+float(self.energy_abso))
              try:
                  text_input =  float(txt)- float(self.pos_now)
                  self.table_motor.setItem(row+1,col,QTableWidgetItem(str('%.2f'%text_input))) 
              except:
                  self.messagebox = QMessageBox.critical(self,'提示','please check the input', QMessageBox.Yes|QMessageBox.Yes)
              
            else:
              try:
                  text_input = float(txt)+ float(self.pos_now)
                  self.table_motor.setItem(row-1,col,QTableWidgetItem(str('%.2f'%text_input)))
              except:
                  self.messagebox = QMessageBox.critical(self,'提示','please check the input', QMessageBox.Yes|QMessageBox.Yes)
             
        if (col == 2 or col == 3):
            if (row % 2 == 0):
               self.table_motor.setItem(row+1,col,QTableWidgetItem(str('%.2f'% float(txt))))
            else:
               self.table_motor.setItem(row-1,col,QTableWidgetItem(str('%.2f'% float(txt)))) 
        self.sig_change += 1
if __name__ == '__main__':
    #app = QApplication(sys.argv)
    #w = MainWindow()
    #sys.exit(app.exec_())
    app=QApplication(sys.argv)
    desktop = app.desktop()
    screen_size = desktop.screenGeometry()
    width = screen_size.width()
    height = screen_size.height() 
    demo = M_widget(1,200)#MainWindow(width,height)
    demo.show()
    #app.setStyleSheet(qdarkstyle.load_stylesheet(palette=LightPalette()))
    sys.exit(app.exec_())




from PyQt5.QtWidgets import QWidget,QHBoxLayout,QTableWidget,QPushButton,QApplication,QVBoxLayout,QTableWidgetItem,QCheckBox,QAbstractItemView,QHeaderView,QLabel,QFrame,QLineEdit,QComboBox,QGridLayout,QGroupBox,QMainWindow,QRadioButton, QToolBar
from PyQt5 import QtWidgets,QtCore
from PyQt5.QtCore import pyqtSignal, QPoint, Qt, QSize, QRectF, QPointF
from PyQt5.QtGui import  QFont,QColor, QPixmapCache, QIcon, QPen
#from faker import Factory
import random
import sys
import pyqtgraph
import pyqtgraph as pg
from pyqtgraph import GraphicsLayoutWidget, mkPen, mkBrush, ImageView
from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph.graphicsItems.ViewBox import ViewBox
from collections import defaultdict
from pyqtgraph import Point, GraphicsObject
from numpy import errstate,isneginf,array





ZOOMING = 1
SELECT = 2
SELECT_POLYGON = 3
PANNING = 4

SELECT_SQUARE = 123
SELECT_POLYGON = 124

# view types
INDIVIDUAL = 0
AVERAGE = 1

# selections
SELECTNONE = 0
SELECTONE = 1
SELECTMANY = 2

MAX_INSTANCES_DRAWN = 1000
MAX_THICK_SELECTED = 10
NAN = float("nan")

# distance to the first point in pixels that finishes the polygon
SELECT_POLYGON_TOLERANCE = 10

COLORBREWER_SET1 = [(228, 26, 28), (55, 126, 184), (77, 175, 74), (152, 78, 163), (255, 127, 0),
                    (255, 255, 51), (166, 86, 40), (247, 129, 191), (153, 153, 153)]



#class imageview_widget(ImageView):
#    def __init__(self):
#        super().__init__()
#        self.normRoi.setPen('r')


#class MyROI(pyqtgraph.ROI):
class MyROI(pg.ROI):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for a, b in [[(0, 0), (1, 1)], [(0, 1), (1, 0)],
            [(0, 0.5), (1, 0.5)], [(0.5, 0), (0.5, 1)]]:
            self.addScaleHandle(a, b)
            self.addScaleHandle(b, a)

    def getXywh(self):
        return tuple(int(x) for x in tuple(self.pos()) + tuple(self.size()))

    def setXywh(self, xywh):
        self.setPos(*xywh[:2], finish = False)
        self.setSize(xywh[2:], finish = False)

class MyImageItem(pyqtgraph.ImageItem):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.first = True

    def setImage(self, image = None, autoLevels = None, **kwargs):
        if autoLevels is None:
            autoLevels = self.first
        super().setImage(image, autoLevels, **kwargs)
        if image is not None:
            self.first = False

class MyImageView(pyqtgraph.GraphicsLayout):
    def __init__(self, *args, view = None, imageItem = None, **kwargs):
        super().__init__(*args, **kwargs)
        self.doStage(view, imageItem)
        self.addItem(self.view, row = 0, col = 0)
        self.addItem(self.lut, row = 0, col = 1)

    def doStage(self, view, imageItem):
        self.view = view or pyqtgraph.PlotItem()
        self.view.setAspectLocked(True)
        self.view.invertY()
        self.image = imageItem or MyImageItem()
        self.view.addItem(self.image)
        self.lut = pyqtgraph.HistogramLUTItem()
        self.lut.setImageItem(self.image)
        self.setShift(0, 0)

    def setShift(self, x, y):
        self.shift = x, y
        self.image.setTransform(QtGui.QTransform().translate(x, y))

    def setImage(self, image, autoRange = True, **kwargs):
        self.image.setImage(image, **kwargs)
        if autoRange:
            self.view.autoRange()

class ProjectImage(MyImageView):
    def __init__(self, *args, view = None,
        imageItem = None, pen = "b", **kwargs):
        pyqtgraph.GraphicsLayout.__init__(self, *args, **kwargs)
        self.doStage(view, imageItem)
        self.addItem(self.view, row = 1, col = 1)
        self.addItem(self.hplot, row = 0, col = 1)
        self.addItem(self.vplot, row = 1, col = 0)
        self.addItem(self.lut, row = 0, col = 2, rowspan = 2, colspan = 1)
        self.vplot.invertX()
        self.vplot.invertY()
        self.hproj.setPen(pen)
        self.vproj.setPen(pen)
        for item, show in [(self.view, ["left", "top"]),
            (self.hplot, ["left"]), (self.vplot, ["top"])]:
            for axis in ["left", "right", "top", "bottom"]:
                item.showAxis(axis, axis in show)

    def doStage(self, *args, **kwargs):
        super().doStage(*args, **kwargs)
        self.hplot = pyqtgraph.PlotItem()
        self.hplot.setXLink(self.view)
        self.hproj = self.hplot.plot()
        self.vplot = pyqtgraph.PlotItem()
        self.vplot.setYLink(self.view)
        self.vproj = self.vplot.plot()
        self.hplot.setMaximumHeight(80)
        self.hplot.setMinimumHeight(24)
        self.vplot.setMaximumWidth(80)
        self.vplot.setMinimumWidth(24)

    def setImage(self, image, **kwargs):
        super().setImage(image, **kwargs)
        xs, ys = numpy.arange(image.shape[1]), numpy.arange(image.shape[0])
        self.hproj.setData(self.shift[0] + xs, image.sum(0))
        self.vproj.setData(image.sum(1), self.shift[1] + ys)

class FxBpmPlot(pyqtgraph.GraphicsView):
    def __init__(self, model, titles, parent = None, mtyps = ({}, {})):
        super().__init__(parent)
        self.timer, self.delay = QtCore.QTimer(), 100
        self.timer.setSingleShot(True)
        self.ci = AlignedLines()
        self.ci.doStage(titles)
        self.setCentralItem(self.ci)
        self.timer.timeout.connect(lambda: self.submit("bpm_idle"))
        #self.sbind(model, mtyps, ["bpm_idle"])
        #self.nbind(mtyps, ["lines"])

    def on_lines(self, lines):
        for i, line in enumerate(self.ci.lines):
            line.setData(lines[0], lines[i + 1])
        self.timer.start(self.delay)

class FxBpmImage(pyqtgraph.GraphicsView):
    def __init__(self, model, parent = None, mtyps = ({}, {})):
        super().__init__(parent)
        self.timer, self.delay = QtCore.QTimer(), 100
        self.xywh = self.xs = self.ys = self.crop = self.mode = None
        self.timer.setSingleShot(True)
        self.ci = ProjectImage()
        self.roi = MyROI((0, 0))
        self.roi.setZValue(10)
        self.ci.view.addItem(self.roi)
        self.setCentralItem(self.ci)
        self.on_roi = self.roi.setXywh
        self.ci.image.hoverEvent = self.submit_hover
        self.timer.timeout.connect(lambda: self.submit("img_idle"))
        self.roi.sigRegionChangeFinished.connect\
            (lambda: self.submit("roi", self.roi.getXywh()))
        #self.sbind(model, mtyps, ["roi", "hover", "img_idle"])
        #self.nbind(mtyps, ["mode", "crop", "img", "roi"])

    def on_mode(self, mode):
        self.mode = mode
        self.roi.setVisible(mode == "open")

    def on_crop(self, xywh):
        self.ci.setShift(*xywh[:2])
        self.wh, roi = xywh[2:], xywh2roi(xywh)
        self.crop = lambda img: roi_slice(img, roi)

    def on_img(self, img):
        self.ci.setImage(self.crop(img))
        self.timer.start(self.delay)

    def submit_hover(self, ev):
        if ev.isExit():
            return
        i, j = numpy.array(self.ci.shift) + \
            numpy.clip(ev.pos(), 0, self.wh).astype(int)
        self.submit("hover", (i, j))



class label_widget(QLabel):
    def __init__(self,txt):
        super().__init__()
        font = QFont('Times New Roman', 12)
        #font.setBold(True)
        self.setFont(font)
        self.setFixedSize(120,50)
        self.setText(txt)
        self.setAlignment(QtCore.Qt.AlignCenter|QtCore.Qt.AlignVCenter)
class button_widget(QPushButton):
    def __init__(self,txt):
        super().__init__()
        font = QFont('Times New Roman', 12)
        #font.setBold(True)
        self.setFont(font)
        self.setFixedSize(120,50)
        self.setText(txt)

class InteractiveViewBox(ViewBox):
    def __init__(self, graph):
        ViewBox.__init__(self, enableMenu=False)
        self.setMenuEnabled(False)
        #self.gragh = graph
        self.setMouseMode(self.PanMode)
        self.zoomstartpoint = None
        self.current_selection = None
        self.action = PANNING
        self.y_padding = 0.0
        self.x_padding = 0

        # line for marking selection
        self.selection_line = pg.PlotCurveItem()
        self.selection_line.setPen(pg.mkPen(color=QColor('black'), width=2))
        self.selection_line.setZValue(1e9)
        self.selection_line.hide()
        self.addItem(self.selection_line, ignoreBounds=True)

        # yellow marker for ending the polygon
        self.selection_poly_marker = pg.ScatterPlotItem()
        self.selection_poly_marker.setPen(pg.mkPen(color=QColor('yellow'), width=2))
        self.selection_poly_marker.setSize(SELECT_POLYGON_TOLERANCE * 2)
        self.selection_poly_marker.setBrush(None)
        self.selection_poly_marker.setZValue(1e9 + 1)
        self.selection_poly_marker.hide()
        self.selection_poly_marker.mouseClickEvent = lambda x: x  # ignore mouse clicks
        self.addItem(self.selection_poly_marker, ignoreBounds=True)

        # self.sigRangeChanged.connect(self.resized)
        # self.sigResized.connect(self.resized)
        self.tiptexts = None

    def mouseMovedEvent(self, ev):  # not a Qt event!
        if self.action == ZOOMING and self.zoomstartpoint:
            pos = self.mapFromView(self.mapSceneToView(ev))
            self.updateScaleBox(self.zoomstartpoint, pos)

    def cancel_zoom(self):
        self.setMouseMode(self.PanMode)
        self.rbScaleBox.hide()
        self.zoomstartpoint = None
        self.action = PANNING
        self.unsetCursor()

    def set_mode_panning(self):
        self.cancel_zoom()
        self.enableAutoRange()

    def set_mode_zooming(self):
        self.set_mode_panning()
        self.setMouseMode(self.RectMode)
        self.action = ZOOMING
        self.setCursor(Qt.CrossCursor)

    def enableAutoRange(self, axis=None, enable=True, x=None, y=None):
        super().enableAutoRange(axis=axis, enable=False, x=x, y=y)

    def enableAutoRangeTrue(self, axis=None, enable=True, x=None, y=None):  ##########
        super().enableAutoRange(axis=axis, enable=True, x=x, y=y)


class InteractiveViewBoxC(InteractiveViewBox):
    def wheelEvent(self, ev, axis=None):
        # separate axis handling with modifier keys
        if axis is None:
            axis = 1 if ev.modifiers() & Qt.ControlModifier else 0
        super().wheelEvent(ev, axis=axis)


        



#!/usr/bin/python3
from PyQt5.QtWidgets import QWidget,QHBoxLayout,QTableWidget,QPushButton,QApplication,QVBoxLayout,QTableWidgetItem,QCheckBox,QAbstractItemView,QHeaderView,QLabel,QFrame,QLineEdit,QComboBox,QGridLayout,QGroupBox,QMainWindow,QRadioButton, QToolBar
from PyQt5 import QtWidgets,QtCore
from PyQt5.QtCore import pyqtSignal, QPoint, Qt, QSize, QRectF, QPointF
from PyQt5.QtGui import  QFont,QColor, QPixmapCache, QIcon, QPen
import random
import sys
import pyqtgraph as pg
from pyqtgraph import GraphicsLayoutWidget, mkPen, mkBrush
from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph.graphicsItems.ViewBox import ViewBox
from .QWidget_widget import FxBpmImage,MyImageItem,MyImageView,MyROI,InteractiveViewBoxC
from  .CurvePlot_widget import CurvePlot
import numpy as np
from .math_func import math_func
import datetime
from PIL import Image
class image_show(QWidget):
    def __init__(self,n):
        super().__init__()
        pg.setConfigOption('background', 'w')
        pg.setConfigOption('foreground', 'k')
        pg.setConfigOptions(imageAxisOrder='row-major')
        #screen_size = desktop.screenGeometry()
        #width = screen_size.width()
        #height = screen_size.height()
        #self.width = int(width/2)
        #self.height = int(height*0.8)
        self.math_func = math_func()
        self.y_axis_name = ['cent_x','cent_y','cent','sum']
        self.plot_items = []
        #self.vbox_plt = QVBoxLayout()
        self.vbox_plt = QGridLayout()
        self.plotimg_item = pg.PlotItem()
        self.ADDROI = True

        self.y_axis_get = False
        
        self.win_image2D = pg.GraphicsLayoutWidget()
        self.img = pg.ImageItem()
        self.p1 = self.win_image2D.addPlot()#(title='Plot-2D(面探测器图像)')
        self.p1.addItem(self.img)
        self.p1.setLabel('bottom','X')
        self.p1.setLabel('left','Y')
        self.hist = pg.HistogramLUTItem()
        self.hist.setImageItem(self.img)
        self.hist.autoHistogramRange = True
        self.win_image2D.addItem(self.hist)
        
        self.win_image2D.setMouseTracking(True)
        #self.set_level_edit()
        def mousePressEvent(event):
            if self.ADDROI:
                if event.button() == Qt.LeftButton:
                    self.ROI_add()
                    self.ADDROI = False
        self.win_image2D.scene().sigMouseClicked.connect(mousePressEvent)
        

        self.n_index = n
        for j in range(2):
            for i in range(self.n_index-1):
                self.plot_item = CurvePlot()
                if (j==1):
                   if (i ==2):
                      self.vbox_plt.addWidget(self.plot_item,i,j,1,1)
                      self.plot_item.set_axis_name("M",self.y_axis_name[-1])
                      self.plot_items.append(self.plot_item)
                   else:
                      self.vbox_plt.addWidget(self.win_image2D,i,j,2,1)
                else:
                   self.plot_items.append(self.plot_item)
                   self.plot_item.set_axis_name("M",self.y_axis_name[i])
                   self.vbox_plt.addWidget(self.plot_item,i,j,1,1)
        
        self.save_path = "/home/mamba/FS_tif"
        self.image_show = None
        self.show_range = None
        self.auto_level = True
        self.roi = None
        self.hbox = QHBoxLayout()
        #vbox.setContentsMargins(0, 0, 0, 0)
        #vbox.addWidget(self.navbar)
        self.hbox.addLayout(self.vbox_plt,1)
        #self.hbox.addWidget(self.image_view,1)
        #self.hbox.addLayout(self.p1)
        #self.hbox.addWidget(self.win_image2D,1)
        #self.setLayout(vbox)
        groupbox1 = QGroupBox("")
        groupbox1.setLayout(self.hbox)
        twogroup = QHBoxLayout()
        twogroup.addWidget(groupbox1)
        self.setLayout(twogroup)
        #self.show()
    def clear_show(self):
        self.x_M = True
        self.sum_roi = []
        self.cent_x_pos = []
        self.cent_y_pos = []
        self.cent_pos = []
        self.x_axis = []
        self.y_axis = []
         
        self.tot_sum_roi = []
        self.tot_cent_x_pos = []
        self.tot_cent_y_pos = []
        self.tot_cent_pos = []
        self.tot_x_axis = []
        self.tot_y_axis = []
        
        for i in range(self.n_index):
            self.plot_items[i].plot.clear()
    def getXywh(self):
        self.show_range = [int(self.roi.pos()[0]),int(self.roi.pos()[0]+self.roi.size()[0]),int(self.roi.pos()[1]),int(self.roi.pos()[1]+self.roi.size()[1])]
        #self.show_range = (tuple((x) for x in tuple(self.roi.pos()) + tuple(self.roi.size())))
        print(self.show_range)
        self.roi_image()
    def roi_image(self):
        self.roi_img = self.image[self.show_range[0]:self.show_range[1],self.show_range[2]:self.show_range[3]]
        #print(len(self.roi_img))
        return(np.sum(self.roi_img))
    def update_plot(self,sum_roi,cent_x_pos,cent_y_pos,cent_pos,x):
        #print(sum_roi,cent_x_pos,cent_y_pos,cent_pos,x)
        self.plot_items[3].plot.plot(x,sum_roi,pen=pg.mkPen(color=(0,0,0),width=3),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=10,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        self.plot_items[0].plot.plot(x,cent_x_pos,pen=pg.mkPen(color=(0,0,0),width=3),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=10,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        self.plot_items[1].plot.plot(x,cent_y_pos,pen=pg.mkPen(color=(0,0,0),width=3),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=10,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        self.plot_items[2].plot.plot(x,cent_pos,pen=pg.mkPen(color=(0,0,0),width=3),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=10,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
    def update_image_show(self, image):
        self.hist = pg.HistogramLUTItem()
        self.hist.setImageItem(self.img)
        self.image = np.array(image)
        self.image_show = True
        image_max = np.max(self.image)
        image_mean = np.mean(self.image)
        self.img.setImage(self.image,autoRange=True,autoHistogramRange=True)
    def update_image_data(self, image):
        self.image = np.array(image)
        self.image_show = True
        #image_max = np.max(self.image)
        #image_mean = np.mean(self.image)
        #log_image = np.log10(self.image)
        #log_image = self.image
        #self.img.setImage(log_image,autoRange=True,autoHistogramRange=True)
        if self.roi:
            self.getXywh()
            sum_roi = self.roi_image()
            cent_x_pos,cent_y_pos,cent_pos = self.math_func.cent_w(self.roi_img)
        else:
            sum_roi = np.sum(self.image)
            cent_x_pos,cent_y_pos,cent_pos = self.math_func.cent_w(self.image)
        #print(cent_x_pos,cent_y_pos,cent_pos)
        self.sum_roi.append(sum_roi)
        self.cent_x_pos.append(cent_x_pos)
        self.cent_y_pos.append(cent_y_pos)
        self.cent_pos.append(cent_pos)
        self.tot_sum_roi.append(sum_roi)
        self.tot_cent_x_pos.append(cent_x_pos)
        self.tot_cent_y_pos.append(cent_y_pos)
        self.tot_cent_pos.append(cent_pos)
        
        if self.x_M:
           self.x_axis = list(range(len(self.sum_roi)))
           self.tot_x_axis = list(range(len(self.tot_sum_roi)))


        tot_data = [self.tot_cent_x_pos,self.tot_cent_y_pos,self.tot_cent_pos,self.tot_sum_roi]
        for i in range(len(tot_data)):
            a,b = self.get_max_min(self.tot_x_axis,tot_data[i])
            if self.y_axis_get :
                c,d = self.get_max_min(self.tot_y_axis,tot_data[i])
                labels = 'max= '+'('+str(a[0])+','+str(c[0])+','+str(a[1])+')'+', min ='+'('+str(b[0])+','+str(d[0])+','+str(b[1])+')'
            else:
                labels = 'max= '+str(a)+', min ='+str(b)
            self.plot_items[i].max_min_label.setText(labels)
        self.update_2d_image()
        self.update_plot(self.sum_roi,self.cent_x_pos,self.cent_y_pos,self.cent_pos,self.x_axis)
        self.save_tif()
    def update_motor_detector(self,motor_x,detector):
        self.name_detector = detector
        self.name_motor_x = motor_x
        #self.name_motor_y = motor_y
    def update_2d_motor_detector(self,motor_y,motor_x,detector):
        self.name_detector = detector
        self.name_motor_x = motor_x
        self.name_motor_y = motor_y
        print(self.name_detector,self.name_motor_x,self.name_motor_y)
    def update_step_y(self,step_y):
        self.step_y = float(step_y)
    def update_plt_y(self,y):
        self.y_axis_get = True
        #print(y)
        self.y_axis.append(y)
        self.tot_y_axis.append(y)
              #self.y_axis = []
    def update_2d_image(self):
        if (len(self.y_axis)>=2):
           if ((self.y_axis[-1]-self.y_axis[-2])>= 0.5*self.step_y):
              self.sum_roi = [self.sum_roi[-1]]
              self.cent_x_pos = [self.cent_x_pos[-1]]
              self.cent_y_pos = [self.cent_y_pos[-1]]
              self.cent_pos = [self.cent_pos[-1]]
              self.x_axis = [self.x_axis[-1]]
       
    def update_image_x(self,x):
        self.x_M = False
        self.x_axis.append(x)
        self.tot_x_axis.append(x)
        #return(self.x_axis)

    def get_max_min(self,list_x,list_y):
        max_a = list_y.index(max(list_y))
        min_a = list_y.index(min(list_y))
        #print(max_a)
        
        max_pos = (float("%.4f" % float(list_x[max_a])), float( "%.4f" % list_y[max_a]))
        min_pos = (float("%.4f" %list_x[min_a]), float( "%.4f" % list_y[min_a]))
        #print(max_pos,min_pos)
        return(max_pos,min_pos)
    def ROI_add(self):
        #print(self.image)
        if self.image_show:
           self.pixel_pos_x = int(self.image.shape[0] *0.4)
           self.pixel_pos_y = int(self.image.shape[1] *0.4)
           self.pixel_size_x = int(self.image.shape[0] *0.1)
           self.pixel_size_y = int(self.image.shape[1] *0.1)
        else:
           self.pixel_pos_x = 0
           self.pixel_pos_y = 0
           self.pixel_size_x = 10
           self.pixel_size_y = 10
        self.roi = MyROI([self.pixel_pos_x, self.pixel_pos_y],[self.pixel_size_x, self.pixel_size_y],maxBounds=QtCore.QRectF(-1000000, -10000000, 1000000000000, 1000000000),pen = pg.mkPen((3,3), width=5.), removable = True)
        self.roi.sigRegionChangeFinished.connect(lambda: self.getXywh())
        self.p1.addItem(self.roi)       
        self.p1.update()
    def update_save_path(self, save_path):
        self.save_path = save_path
    def save_tif(self):
        time_now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        save_path = self.save_path
        #print(save_path)
        #save_file_x = str(self.x_axis[-1])
        #save_file_y = str(self.y_axis[-1]
        #if self.name_motor_x == 0:
        #    self.name_motor_x = 'count'
        try:   
              if self.y_axis:
                  save_file = save_path+ "/" +self.name_motor_x+ str(self.x_axis[-1])+" "+self.name_motor_y + str(self.y_axis[-1])+self.name_detector+" "+time_now
              else:
                  save_file = save_path + "/"+self.name_motor_x+str(self.x_axis[-1])+self.name_detector +" "+time_now
              #print(save_file)
              with open("/home/mamba/FS_tif/center_Fs.txt","a") as f:
                  if self.y_axis:
                      f.writelines(self.name_motor_x +"%13.8f" % self.x_axis[-1]+" "+self.name_motor_y +"%13.8f" % self.y_axis[-1]+" "+self.name_detector +" "+"%13.8f" % self.sum_roi[-1]+" "+"%13.8f" % self.cent_x_pos[-1]+" "+"%13.8f" % self.cent_y_pos[-1]+" "+"%13.8f" % self.cent_pos[-1]+" "+time_now)
                  else:
                      f.writelines(self.name_motor_x +"%13.8f" % self.x_axis[-1]+" "+self.name_detector +"%13.8f" % self.sum_roi[-1]+" "+"%13.8f" % self.cent_x_pos[-1]+" "+"%13.8f" % self.cent_y_pos[-1]+" "+"%13.8f" % self.cent_pos[-1]+" "+time_now)
                  #f.writelines("%11.8f" % float(save_file_x)+"%15.8f" %  float(save_file_y)+"     "+time_now)
                  f.writelines("\n")
              try:
                  Image.fromarray(self.image).save(save_file+".tif")
                  Image.fromarray(self.image).save(save_file+".png")
              #try:
              #    Image.fromarray(self.image).save(save_file+".tif")
              except:
                  print("error")
        except:
              self.name_motor_x = 'count '
              self.name_detector = 'FS  '
              if self.y_axis:
                  save_file = save_path+ "/" +self.name_motor_x+ str(self.x_axis[-1])+" "+self.name_motor_y + str(self.y_axis[-1])+self.name_detector+" "+time_now
              else:
                  save_file = save_path + "/"+self.name_motor_x+str(self.x_axis[-1])+self.name_detector +" "+time_now
              #print(save_file)
              with open("/home/mamba/FS_tif/center_Fs.txt","a") as f:
                  if self.y_axis:
                      f.writelines(self.name_motor_x +"%13.8f" % self.x_axis[-1]+" "+self.name_motor_y +"%13.8f" % self.y_axis[-1]+" "+self.name_detector +" "+"%13.8f" % self.sum_roi[-1]+" "+"%13.8f" % self.cent_x_pos[-1]+" "+"%13.8f" % self.cent_y_pos[-1]+" "+"%13.8f" % self.cent_pos[-1]+" "+time_now)
                  else:
                      f.writelines(self.name_motor_x +"%13.8f" % self.x_axis[-1]+" "+self.name_detector +"%13.8f" % self.sum_roi[-1]+" "+"%13.8f" % self.cent_x_pos[-1]+" "+"%13.8f" % self.cent_y_pos[-1]+" "+"%13.8f" % self.cent_pos[-1]+" "+time_now)
                  #f.writelines("%11.8f" % float(save_file_x)+"%15.8f" %  float(save_file_y)+"     "+time_now)
                  f.writelines("\n")
              try:
                  Image.fromarray(self.image).save(save_file+".tif")
                  Image.fromarray(self.image).save(save_file+".png")
              #try:
              #    Image.fromarray(self.image).save(save_file+".tif")
              except:
                  print("error")
if __name__ == '__main__':
    app = QApplication(sys.argv)
    desktop = app.desktop()
    w = image_show(4)
    img = np.array([[random.randrange(10,100,2)
                              for j in range(0,1024)]
                             for i in range(0,1024)])
    w.updata_image2D(img)
    w.show()
    sys.exit(app.exec_())



#!/usr/bin/python3
from PyQt5.QtWidgets import QWidget,QHBoxLayout,QTableWidget,QPushButton,QApplication,QVBoxLayout,QTableWidgetItem,QCheckBox,QAbstractItemView,QHeaderView,QLabel,QFrame,QLineEdit,QComboBox,QGridLayout,QGroupBox,QMainWindow,QRadioButton, QToolBar
from PyQt5 import QtWidgets,QtCore
from PyQt5.QtCore import pyqtSignal, QPoint, Qt, QSize, QRectF, QPointF
from PyQt5.QtGui import  QFont,QColor, QPixmapCache, QIcon, QPen
#from faker import Factory
import random
import sys
import pyqtgraph as pg
from pyqtgraph import GraphicsLayoutWidget, mkPen, mkBrush
from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph.graphicsItems.ViewBox import ViewBox
from .QWidget_widget import label_widget, button_widget
from .mrc_mnc_backend import mrc_mnc

class D_widget(QWidget):
    def __init__(self,index,value_now,mrc,mnc):
        super().__init__()
        #self.table_detector = Table_widget()
        self.mrc_mnc = mrc_mnc(mrc,mnc)
        self.detector_list_read = self.mrc_mnc.get_D_list()
        #print(self.detector_list_read)
        #self.cali_b = None
        #self.cali_k = None
        #self.math_func = math_func()
        #self.motor_list = self.math_func.read_M_list(self.motor_list_read)
        self.detector_combox = QComboBox()
        self.detector_combox.setFixedSize(120, 40)
        self.detector_combox.addItems(self.detector_list_read)
        self.detector_name = label_widget("探测器"+str(index))
        self.detector_now = label_widget("积分时间")
        self.detector_now_lineedit = QLineEdit("0.2")
        self.detector_now_lineedit.setFixedSize(120, 40)
        self.loop_n = label_widget("循环次数")
        self.loop_lineedit = QLineEdit("1")
        self.loop_lineedit.setFixedSize(120, 40)
        #self.btn_roi = button_widget('ROI')
        self.btn_D_start = button_widget('Start')
        self.btn_D_start.setFixedSize(120, 40)
        self.btn_D_pause = button_widget('Pause')
        self.btn_D_pause.setFixedSize(120, 40)
        self.btn_D_end = button_widget('Stop')
        self.btn_D_end.setFixedSize(120, 40)
        self.btn_D_cali = button_widget('calibration')
        self.btn_D_cali.setFixedSize(120, 80)
        self.vbox1 = QVBoxLayout()
        self.vbox1.addWidget(self.detector_name)
        self.vbox1.addWidget(self.detector_combox)
        self.vbox1.addWidget(self.btn_D_start)
        self.vbox2 = QVBoxLayout()
        self.vbox2.addWidget(self.detector_now)
        self.vbox2.addWidget(self.detector_now_lineedit)
        self.vbox2.addWidget(self.btn_D_pause)
        self.vbox3 = QVBoxLayout()
        self.vbox3.addWidget(self.loop_n)
        self.vbox3.addWidget(self.loop_lineedit)
        self.vbox3.addWidget(self.btn_D_end)
        self.hbox = QHBoxLayout()
        self.hbox.addLayout(self.vbox1)
        self.hbox.addLayout(self.vbox2)
        self.hbox.addLayout(self.vbox3)
        #self.hbox.addWidget(self.btn_D_cali)
        self.vbox3 = QVBoxLayout()
        self.vbox3.addLayout(self.hbox)
        #self.vbox3.addWidget(self.table_detector)
        groupbox1 = QGroupBox("探测器")#+str(index))
        groupbox1.setLayout(self.vbox3)
        self.hhbox = QHBoxLayout()
        self.hhbox.addWidget(groupbox1)
        self.setLayout(self.hhbox)
        
        
    def start_count(self,scantype):
        #print(scantype)
        D_time = self.detector_now_lineedit.text()
        D_name = self.detector_combox.currentText()
        D_loop = self.loop_lineedit.text()
        self.mrc_mnc.D_start(D_name, D_time, D_loop, str(scantype))
    def end_count(self):
        self.mrc_mnc.D_end()
    def pause_count(self):
        self.mrc_mnc.D_pause()
    def resume_count(self):
        self.mrc_mnc.D_resume()

if __name__ == '__main__':
    #app = QApplication(sys.argv)
    #w = MainWindow()
    #sys.exit(app.exec_())
    app=QApplication(sys.argv)
    desktop = app.desktop()
    screen_size = desktop.screenGeometry()
    width = screen_size.width()
    height = screen_size.height() 
    demo = D_widget(1,200)#MainWindow(width,height)
    demo.show()
    #app.setStyleSheet(qdarkstyle.load_stylesheet(palette=LightPalette()))
    sys.exit(app.exec_())




import numpy as np
import math
import sys
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QWidget
import random
from PyQt5.QtCore import pyqtSignal
import datetime



class mrc_mnc(QWidget):
    #trigger_oneD_scan_status = pyqtSignal(str)
    #trigger_twoD_scan_status = pyqtSignal(str)
    def __init__(self,mrc,mnc,parent=None):
        super().__init__(parent)
        self.mrc = mrc
        self.mnc = mnc
        self.detector_image = ['D.WhiteFS','D.IntegratedFS', 'D.MonochromaticFS', 'D.HRFS1', 'D.HRFS2']
        self.detector_point = ['电离室']
        self.detector_xbpm = ['D.WhiteXBPM','D.QXBPM']
        #m = {'extra':{'type': 'detector_scan_start', 'mode': 'prepare', 'detector': D_name}}
        #command = "P.grid_scan(["+D_backend_str.strip("''")+",D.ct08],M.BraggTheta,"+ str(theta_start) + ","+str(theta_end) +","+str(n_step)+",md =" + str(m) +")\n"
############################################################################
    def D_set_time(self,D_name,D_time):
        D_name = str(D_name)
        
        if D_name in self.detector_image:
            command_time =  "%go "+ D_name+".cam.acquire_time.set(" + str(D_time) + ").wait()\n"
            self.mrc.do_cmd(command_time)
        else:
            pass
            #command_time =  D_name+".acquire_time.set(" + str(D_time) + ").wait()\n"
        #print(command_time)
    def D_image_cam(self,D_name):
        cmd_monitor =  "%go "+ D_name +".cam.acquire.set(%d).wait()\n" % int(0)
            #self.mrc.req_rep_base("cmd", go = "", cmd = cmd_monitor)
        self.mrc.do_cmd(cmd_monitor)   
###############################################################################
    def D_start(self,D_name,D_time,D_loop,scantype):
        if D_name in self.detector_image:
            cmd_monitor =  "%go "+ D_name +".cam.acquire.set(%d).wait()\n" % int(0)
            #self.mrc.req_rep_base("cmd", go = "", cmd = cmd_monitor)
            self.mrc.do_cmd(cmd_monitor)
        else:
            pass
        self.D_set_time(D_name,D_time)
        #D_name = str(D_name)
        #command_time =  D_name+".acquire_time.set(" + str(D_time) + ").wait()\n"
        #self.mrc.do_cmd(command_time)
        #D_loop = int(D_loop)
        time_now = datetime.datetime.now().strftime("%Y-%m-%d-%H:%M:%S")
        save_path = '/home/mamba/mdw_result/experiment'+time_now
        save_file_name = 'master.nxs'
        m = {'extra':{'hyktype': 'detector_count', 'mode':str(scantype), 'detector': D_name },'filepath_store_dir':save_path,'filepath_store_filename':save_file_name}
        #for i in range (D_loop):
        command_count = "P.count([" + D_name +"], "+ D_loop  + ",md =" + str(m) +")\n"
        self.mrc.req_rep_base("cmd", go = "", cmd = command_count)
    def D_end(self):
        try:
           self.mrc.req_rep("scan/abort")
        except:
           pass
    def D_pause(self):
        self.mrc.req_rep("scan/pause")
    def D_resume(self):
        self.mrc.req_rep("scan/resume")
############################################################################
    def oneD_scan_start(self,M_name,M_start,M_end,M_n_step,D_name,D_exp):
        if D_name in self.detector_image:
            cmd_monitor =  "%go "+ D_name +".cam.acquire.set(%d).wait()\n" % int(0)
            #self.mrc.req_rep_base("cmd", go = "", cmd = cmd_monitor)
            self.mrc.do_cmd(cmd_monitor)
        else:
            pass
        #cmd_monitor =  "%go "+ D_name +".cam.acquire.set(%d).wait()\n" % int(0)
        #self.mrc.req_rep_base("cmd", go = "", cmd = cmd_monitor)
        #self.mrc.do_cmd(cmd_monitor)
        Mone = M_name.replace('.','_',1)
        self.D_set_time(D_name,D_exp)
        #m = {"hyktype": "detector_scan_start" , "frame_count":5000,"scan_type":"stepscan","extra":"scan"}
        #m = {'extra':{'hyktype': 'detector_scan_start', 'mode': 'oneD', 'detector': D_name, 'Motor':Mone}}
        #print(m)
        time_now = datetime.datetime.now().strftime("%Y-%m-%d-%H:%M:%S")
        save_path = '/home/mamba/mdw_result/experiment'+time_now
        save_file_name = 'master.nxs'
        m = {'extra':{'hyktype': 'detector_count', 'mode':'oneD', 'detector': D_name },'filepath_store_dir':save_path,'filepath_store_filename':save_file_name}
        #mdw_req_cmd_gui(cmds = m)
        #print("5$$$")#self.mrc.req_rep_base("cmd", go = "", cmd = command)    
        command = "P.grid_scan(["+str(D_name) +"],"+str(M_name)+"," + str(M_start) + ","+str(M_end) +","+str(M_n_step)+",md ="+ str(m) +")\n"
        #print(command)
        self.mrc.req_rep_base("cmd", go = "", cmd = command) 
    def twoD_scan_start(self,M_name,M_start,M_end,M_n_step,M2_name,M2_start,M2_end,M2_n_step,D_name,D_exp):
        Mone = M_name.replace('.','_',1)
        Mtwo = M2_name.replace('.','_',1)
        #print(Mone,Mtwo)
        self.D_set_time(D_name,D_exp)
        time_now = datetime.datetime.now().strftime("%Y-%m-%d-%H:%M:%S")
        save_path = '/home/mamba/mdw_result/experiment'+time_now
        save_file_name = 'master.nxs'
        #m = {'extra':{'hyktype': 'detector_count', 'mode':str(scantype), 'detector': D_name },'filepath_store_dir':save_path,'filepath_store_filename':save_file_name}
        m = {'extra':{'hyktype': 'detector_scan_start', 'mode': 'twoD', 'detector': D_name, 'Motor':(Mone,Mtwo)},'filepath_store_dir':save_path,'filepath_store_filename':save_file_name}
        command = "P.grid_scan(["+str(D_name) +"],"+str(M_name)+"," + str(M_start) + ","+str(M_end) +","+str(M_n_step)+','+str(M2_name)+"," + str(M2_start) + ","+str(M2_end) +","+str(M2_n_step)+",snake_axes = False, md =" + str(m)+")\n"
        #print(command)
        self.mrc.req_rep_base("cmd", go = "", cmd = command)

###################################################################################

    def oneD_scan_end(self):
        self.mrc.req_rep("scan/abort")
        #self.trigger_oneD_scan_status.emit('oneD_scan_end')
        #print("oneD_scan_end")
    def oneD_scan_pause(self):
        self.mrc.req_rep("scan/pause")
        #self.trigger_oneD_scan_status.emit('oneD_scan_pause')
    def oneD_scan_resume(self):
        self.mrc.req_rep("scan/resume")
        #self.trigger_oneD_scan_status.emit('oneD_scan_resume')
    def twoD_scan_end(self):
        self.mrc.req_rep("scan/abort")
        #self.trigger_twoD_scan_status.emit('twoD_scan_end')
    def twoD_scan_pause(self):
        self.mrc.req_rep("scan/pause")
        #self.trigger_twoD_scan_status.emit('twoD_scan_pause')
    def twoD_scan_resume(self):
        self.mrc.req_rep("scan/resume")
        #self.trigger_twoD_scan_status.emit('twoD_scan_resume')
##########################################################################
    def set_M(self,M_name,M_pos):
        cmd = "%go "+ M_name +".set(%f).wait()\n" % float(M_pos)
        #print(cmd)
        self.mrc.do_cmd("%go "+ M_name +".set(%f).wait()\n" % float(M_pos))
    def stop_M(self, M_name):
        cmd = "%go "+ M_name +".stop() \n"
        self.mrc.do_cmd("%go "+ M_name +".stop() \n")
    def limit_M_pos(self,M_name):
        pos_limit_dic = self.mrc.req_rep("dev/read_configuration", path = M_name)["ret"]
        #print(pos_limit_dic)
        try:
            pos_high = pos_limit_dic[M_name+'.high_limit_travel']['value']
            pos_low = pos_limit_dic[M_name+'.low_limit_travel']['value']
        except:
            pos_high = 10000.0
            pos_low = -10000.0
        return(pos_high,pos_low)
    def get_M_pos(self,M_name):
        cmd = "dev/read"+", path = " + M_name
        try:
            pos_now_dic = self.mrc.req_rep("dev/read", path = M_name)["ret"]
            pos_limit_dic = self.mrc.req_rep("dev/read_configuration", path = M_name)["ret"]
        except:
            pass
            #print("please check M name")
        #print(pos_limit_dic)
        try:
            pos_unit = pos_limit_dic[M_name+'.motor_egu']['value']
        except:
            pos_unit = 'degree'
        try:
            pos_now = pos_now_dic[M_name+'.readback']['value']
        except:
            pos_now = 0.0
        #print(pos_now,pos_unit)
        return(pos_now,pos_unit)
    def get_M_list(self):
        motor_name = self.mrc.req_rep("dev/keys", path = "M")["ret"]
        #detector_name = self.mrc.req_rep("dev/keys", path = "D")["ret"]
        return(motor_name)
    def get_D_list(self):
        detector_name = self.mrc.req_rep("dev/keys", path = "D")["ret"]
        #detector_name = self.mrc.req_rep("dev/keys", path = "D")["ret"]
        return(detector_name)




#!/usr/bin/python3
import configparser
from PyQt5.QtWidgets import QWidget,QHBoxLayout,QTableWidget,QPushButton,QApplication,QVBoxLayout,QTableWidgetItem,QCheckBox,QAbstractItemView,QHeaderView,QLabel,QFrame,QLineEdit,QComboBox,QGridLayout,QGroupBox,QMainWindow,QRadioButton, QToolBar
from PyQt5 import QtWidgets,QtCore
from PyQt5.QtCore import pyqtSignal, QPoint, Qt, QSize, QRectF, QPointF
from PyQt5.QtGui import  QFont,QColor, QPixmapCache, QIcon, QPen
#from faker import Factory
import random
import sys
import numpy as np
import pyqtgraph as pg
from pyqtgraph import GraphicsLayoutWidget, mkPen, mkBrush
from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph.graphicsItems.ViewBox import ViewBox
from collections import defaultdict
from pyqtgraph import Point, GraphicsObject
from numpy import errstate,isneginf,array
from qt_material import apply_stylesheet
#import qdarkstyle
#from qdarkstyle.light.palette import LightPalette
from .D_widget import D_widget
from .M_widget import M_widget
from .CurvePlot_widget import CurvePlot
from .imageshow_widget import image_show
from .xbpm_widget import xbpm_widget
from .QWidget_widget import label_widget, button_widget
from .one_image_scan_widget import one_image_scan
from .two_image_scan_widget import two_image_scan
from .thread_scan import Mythread_SCAN
from .mrc_mnc_backend import mrc_mnc        

#from dim_relayout import dim_relayout
sys.setrecursionlimit(10000)




class dim_relayout(QWidget):
    def __init__(self, parent=None):
        super(dim_relayout,self).__init__(parent)
        #super(MainWindow, self).__init__(parent)
    def detector_one_widget_cor(self):
        detector_now = self.oneDscan.D1.detector_combox.currentText()
        self.detector_one_widget(detector_now)
        self.trigger_one_connect()
    def detector_two_widget_cor(self):
        detector_now = self.twoDscan.D1.detector_combox.currentText()
        self.detector_two_widget(detector_now)
        self.trigger_two_connect()
        #index_now = self.twoDscan.D1.detector_combox.currentIndex()       
        #self.oneDscan.D1.detector_combox.setCurrentIndex(index_now)
    def detector_one_widget_ini(self):
        self.detector_now = 'D.WhiteFS'
    def detector_two_widget_ini(self):
        self.detector_now = 'D.WhiteFS'
        
    def detector_one_widget(self, detector_now):   
        #detector_now = self.oneDscan.D1.detector_combox.currentText()
        if self.detector_now in self.oneDscan.detector_point:
           self.oneDscan.plt_widget.deleteLater()
        elif self.detector_now in self.oneDscan.detector_xbpm:
           self.oneDscan.xbpm_widget.deleteLater()
        elif self.detector_now in self.oneDscan.detector_image:
           self.oneDscan.img_widget.deleteLater()

        if detector_now in self.oneDscan.detector_point:
           self.oneDscan.plt_widget = CurvePlot()
           self.oneDscan.img_hbox.addWidget(self.oneDscan.plt_widget,4)
        elif detector_now in self.oneDscan.detector_xbpm:
           self.oneDscan.xbpm_widget = xbpm_widget(6)
           self.oneDscan.img_hbox.addWidget(self.oneDscan.xbpm_widget,4)
           self.oneDscan.D_cali.clicked.connect(self.oneDscan.xbpm_widget.xbpm_cali)
           self.oneDscan.xbpm_widget.trigger_xbpm_k_b.connect(self.update_k_b)
           self.oneDscan.xbpm_widget.init_k_b(self.k_up,self.b_up,self.k_left,self.b_left)
        elif detector_now in self.oneDscan.detector_image:
           self.oneDscan.img_widget = image_show(4)
           self.oneDscan.img_hbox.addWidget(self.oneDscan.img_widget,4)
        self.detector_now = detector_now
        self.show()

    def detector_two_widget(self, detector_now):   
        #detector_now = self.twoDscan.D1.detector_combox.currentText()
        if self.detector_now in self.twoDscan.detector_point:
           self.twoDscan.plt_widget.deleteLater()
        elif self.detector_now in self.twoDscan.detector_xbpm:
           self.twoDscan.xbpm_widget.deleteLater()
        elif self.detector_now in self.twoDscan.detector_image:
           self.twoDscan.img_widget.deleteLater()

        if detector_now in self.twoDscan.detector_point:
           self.twoDscan.plt_widget = CurvePlot()
           self.twoDscan.img_hbox.addWidget(self.twoDscan.plt_widget,4)
        elif detector_now in self.twoDscan.detector_xbpm:
           self.twoDscan.xbpm_widget = xbpm_widget(6)
           self.twoDscan.img_hbox.addWidget(self.twoDscan.xbpm_widget,4)
           self.twoDscan.xbpm_widget.init_k_b(self.k_up,self.b_up,self.k_left,self.b_left)
        elif detector_now in self.twoDscan.detector_image:
           self.twoDscan.img_widget = image_show(4)
           self.twoDscan.img_hbox.addWidget(self.twoDscan.img_widget,4)
        self.detector_now = detector_now
        self.show()
if __name__ == '__main__':
    #app = QApplication(sys.argv)
    #w = MainWindow()
    #sys.exit(app.exec_())
    app=QApplication(sys.argv)
    desktop = app.desktop()
    screen_size = desktop.screenGeometry()
    width = screen_size.width()
    height = screen_size.height() 
    demo = MainWindow(width,height)
    demo.show()
    #app.setStyleSheet(qdarkstyle.load_stylesheet(palette=LightPalette()))
    sys.exit(app.exec_())



#!/usr/bin/python3
from PyQt5.QtWidgets import QWidget,QHBoxLayout,QTableWidget,QPushButton,QApplication,QVBoxLayout,QTableWidgetItem,QCheckBox,QAbstractItemView,QHeaderView,QLabel,QFrame,QLineEdit,QComboBox,QGridLayout,QGroupBox,QMainWindow,QRadioButton, QToolBar
from PyQt5 import QtWidgets,QtCore
from PyQt5.QtCore import pyqtSignal, QPoint, Qt, QSize, QRectF, QPointF,QVariant
from PyQt5.QtGui import  QFont,QColor, QPixmapCache, QIcon, QPen
#from faker import Factory
import random
import sys
import pyqtgraph as pg
from pyqtgraph import GraphicsLayoutWidget, mkPen, mkBrush
from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph.graphicsItems.ViewBox import ViewBox
from collections import defaultdict
from pyqtgraph import Point, GraphicsObject
from numpy import errstate,isneginf,array
from qt_material import apply_stylesheet
import qdarkstyle
from qdarkstyle.light.palette import LightPalette
from .D_widget import D_widget
from .M_widget import M_widget
from .CurvePlot_widget import CurvePlot
from .imageshow_widget import image_show
from .QWidget_widget import label_widget, button_widget
from .mrc_mnc_backend import mrc_mnc
from .xbpm_widget import xbpm_widget
class save_fil(QWidget):
    def __init__(self,path):
        super().__init__()
        self.btn_save = QtWidgets.QPushButton("Path")
        self.btn_save.setFixedSize(70,70)
        self.btn_load = QtWidgets.QPushButton("Load")
        self.btn_load.setFixedSize(70,70)
        self.lineedit_save = QtWidgets.QLineEdit(path)
        self.lineedit_save.setFixedSize(250,40)
        self.lineedit_save.setReadOnly(True)
        self.hlayout = QtWidgets.QHBoxLayout()
        #self.hlayout.addWidget(self.label_save)
        self.hlayout.addWidget(self.lineedit_save)
        self.hlayout.addWidget(self.btn_save)
        #self.hlayout.addWidget(self.btn_save)
        self.setLayout(self.hlayout)
        #self.btn_load.clicked.connect(self.fil_save)
    def fil_save(self):
        options = QtWidgets.QFileDialog.Options()
        options |=  QtWidgets.QFileDialog.DontUseNativeDialog
        directory = QtWidgets.QFileDialog.getExistingDirectory(None,'open file dialog','/home/ke/Downloads/IHEP/BF',options=options)
        self.lineedit_save.setText(directory)
  
class two_image_scan(QWidget):
    trigger_twoD_scan_status = pyqtSignal(str)
    trigger_twoD_My_step = pyqtSignal(str)
    
    def __init__(self,mrc,mnc):
        super().__init__()
        #self.table_motor = Table_widget()
        self.msg = None
        self.scan_paused = False
        self.plt_widget = CurvePlot()
        self.img_widget = image_show(4)
        self.xbpm_widget = xbpm_widget(6)
        self.mrc_mnc = mrc_mnc(mrc,mnc)
        self.scan_paused = False
        self.count_paused = False
        self.save_path = save_fil("/home/mamba/FS_tif")
        font = QFont('Times New Roman',15)
        self.motor_list = {'slit1': ['M1.x','M1.y','M1.z'],'slit2':['M2.x','M2.y','M2.z'],'slit3':['M3.x','M3.y','M3.z']}
        self.scan_label = QLabel("scan type:    ")
        self.scan_label.setFont(font)
        self.scan_label.setFixedSize(150,40)
       
        self.detector_image = ['D.WhiteFS','D.IntegratedFS', 'D.MonochromaticFS', 'D.HRFS1', 'D.HRFS2']
        self.detector_point = ['电离室']
        self.detector_xbpm = ['D.WhiteXBPM','D.QXBPM']
        #self.motor_label = label_widget("设备")
        #self.motor_combox = QComboBox(self)
        #self.detector_combox = QComboBox(self)
        #self.motor_pv_combox = QComboBox(self)
        #self.motor_combox.setFixedSize(100, 40)
        #self.motor_pv_combox.setFixedSize(100, 40)
        #self.detector_combox.setFixedSize(100, 40)
        self.one_Dscan = QRadioButton("1D scan")
        self.two_Dscan = QRadioButton("2D scan")
        self.one_Dscan.setFixedSize(150,40)
        self.one_Dscan.setFont(font)
        self.two_Dscan.setFixedSize(150,40)
        self.two_Dscan.setFont(font)
        self.two_Dscan.setChecked(True)
        #self.motor_vbox = QVBoxLayout()
        #self.motor_vbox.addWidget(self.motor_label)
        #self.motor_vbox.addWidget(self.motor_combox)
        
        self.scan_type_vbox =  QHBoxLayout()
        self.scan_type_vbox.addWidget(self.scan_label)
        self.scan_type_vbox.addWidget(self.one_Dscan)
        self.scan_type_vbox.addWidget(self.two_Dscan)
        self.btn_scan_start = button_widget('Start')
        self.btn_scan_start.setFixedSize(120, 100)
        self.btn_scan_pause = button_widget('Pause')
        self.btn_scan_pause.setFixedSize(120, 100)
        self.btn_scan_end = button_widget('End')
        self.btn_scan_end.setFixedSize(120, 100)
        self.hbtnbox = QHBoxLayout()
        self.hbtnbox.addWidget(self.btn_scan_start)
        self.hbtnbox.addWidget(self.btn_scan_pause)
        self.hbtnbox.addWidget(self.btn_scan_end)
        
       
        self.hbox = QHBoxLayout()
        #self.hbox.addLayout(self.motor_vbox)
        self.hbox.addLayout(self.scan_type_vbox)
        groupbox = QGroupBox("扫描类型")
        groupbox.setLayout(self.hbox)
        self.scanhbox = QHBoxLayout()
        self.scanhbox.addWidget(groupbox)
        #self.hbox.addWidget(self.one_Dscan)
        #self.hbox.addWidget(self.two_Dscan)
       
        self.M1 = M_widget(1,200,mrc,mnc)
        self.M2 = M_widget(2,200,mrc,mnc)
        self.gvbox = QVBoxLayout()
        self.gvbox.addWidget(self.M1)
        self.gvbox.addWidget(self.M2)
        self.gvbox.addLayout(self.hbtnbox)
        self.groupbox1 = QGroupBox("电机")
        self.groupbox1.setLayout(self.gvbox)
        
        self.D1 = D_widget(1,200,mrc,mnc)
        #self.motor_combox.addItems(self.motor_list.keys())
        #self.M1.motor_pv_combox.addItems(self.motor_list['slit1'])
        #self.M2.motor_pv_combox.addItems(self.motor_list['slit1'])
        #self.D1.detector_combox.addItems(self.detector_image)
        #self.D1.detector_combox.addItems(self.detector_point)
        #self.D1.detector_combox.addItems(self.detector_xbpm)
        self.group_hbox = QVBoxLayout()
        self.group_hbox.addItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding))
        self.group_hbox.addLayout(self.scanhbox)
        self.group_hbox.addItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding))
        self.group_hbox.addWidget(self.groupbox1)
        self.group_hbox.addItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding))
        self.group_hbox.addWidget(self.D1)
        self.group_hbox.addWidget(self.save_path)
        #self.group_hbox.addWidget(self.M2)
        #self.group_hbox = QHBoxLayout()
        #self.group_hbox.addLayout(self.vbox)
        
        
        self.group_hbox.addItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding))
        #self.vbox.addWidget(self.plt_widget)
        
        #groupbox1 = QGroupBox("")
        #groupbox1.setLayout(self.group_hbox)
        self.img_hbox = QHBoxLayout()
        self.img_hbox.addLayout(self.group_hbox)
        self.img_hbox.addWidget(self.img_widget)
        #self.setLayout(twogroup)        
        self.img_hbox.setStretch(0, 1)
        self.img_hbox.setStretch(1, 4)
        self.img_hbox.setSpacing(0)
        #self.setLayout(self.img_hbox)
        self.motor_1_pv()
        self.motor_2_pv()
        self.M1.motor_combox.currentIndexChanged.connect(self.motor_1_pv)
        self.M2.motor_combox.currentIndexChanged.connect(self.motor_2_pv)
        self.M1.motor_pv_combox.currentIndexChanged.connect(self.motor_pv1_disable)
        #self.M2.motor_pv_combox.currentIndexChanged.connect(self.motor_pv2_disable)
        self.motor_pv1_disable()
        self.btn_scan_start.clicked.connect(self.scan_start)
        self.btn_scan_end.clicked.connect(self.scan_end)
        self.btn_scan_pause.clicked.connect(self.scan_pause)
        self.D1.btn_D_start.clicked.connect(lambda x: self.start_count(str('twoD')))
        self.D1.btn_D_pause.clicked.connect(self.pause_count)
        self.D1.btn_D_end.clicked.connect(self.end_count)
        #self.save_path.btn_load.clicked.connect(self.img_widget.load_plot)
    def btn_status(self,msg):
        #print("******",msg)
        if msg == 'twoD_scan_stop':
           #self.D1.btn_D_start.setEnabled(True)
           #self.D1.btn_D_pause.setEnabled(False)
           #self.D1.btn_D_end.setEnabled(False)
           self.btn_scan_start.setEnabled(True)
           self.btn_scan_end.setEnabled(False)
           self.btn_scan_pause.setEnabled(False)
        if  msg == 'twoD_scan_start':
           if self.msg == 'twoD_scan_end' or self.msg == 'twoD_scan_pause':
               self.btn_scan_start.setEnabled(True)
               self.btn_scan_end.setEnabled(False)
               self.btn_scan_pause.setEnabled(False)
           else:
               self.btn_scan_start.setEnabled(False)
               self.btn_scan_end.setEnabled(True)
               self.btn_scan_pause.setEnabled(True)
        if  msg == 'twoD_scan_resume':
           self.btn_scan_start.setEnabled(False)
           self.btn_scan_end.setEnabled(True)
           self.btn_scan_pause.setEnabled(True)
        if  msg == 'twoD_scan_end':
           self.btn_scan_start.setEnabled(True)
           self.btn_scan_end.setEnabled(False)
           self.btn_scan_pause.setEnabled(False)
        if  msg == 'twoD_scan_pause':
           self.btn_scan_start.setEnabled(True)
           self.btn_scan_end.setEnabled(False)
           self.btn_scan_pause.setEnabled(False)
        self.msg = msg



    def motor_pv_update(self):
        self.M1.update_pos_now()
        self.M1.init_item()
        self.M2.update_pos_now()
        self.M2.init_item()
    def motor_1_pv(self):
        self.M1.motor_pv_combox.clear()
        #self.M2.motor_pv_combox.clear()
        motor_now = self.M1.motor_combox.currentText()
        motor_pv = self.M1.motor_list[motor_now]
        self.M1.motor_pv_combox.addItems(motor_pv)
        self.M1.init_item()
        #self.M2.motor_pv_combox.addItems(motor_pv)
    def motor_2_pv(self):
        #self.M1.motor_pv_combox.clear()
        self.M2.motor_pv_combox.clear()
        motor_now = self.M2.motor_combox.currentText()
        motor_pv = self.M2.motor_list[motor_now]
        #self.M1.motor_pv_combox.addItems(motor_pv)
        self.M2.motor_pv_combox.addItems(motor_pv)
        self.M2.init_item()
        
    def motor_pv1_disable(self):
        self.motor_2_pv()
        if (self.M1.motor_combox.currentText() == self.M2.motor_combox.currentText()):
            motor_1 = self.M1.motor_pv_combox.currentText()
            index = self.M2.motor_pv_combox.findText(motor_1)
            if index == 0:
                self.M2.motor_pv_combox.setCurrentIndex(index+1)
            self.M2.motor_pv_combox.setItemData(index,QVariant(0),Qt.UserRole-1)
            #self.M2.motor_pv_combox.setCurrentIndex(index)

    def scan_start(self):
        M_name = {}
        M_para = {}
        if self.scan_paused:
            self.scan_resume()
        else:
            try:
               self.img_widget.clear_show()
            except:
               pass
            try:
               self.xbpm_widget.clear_show()
            except:
               pass
        #self.btn_scan_start.setEnabled(False)
        #self.btn_scan_end.setEnabled(True)
        #self.btn_scan_pause.setEnabled(True)
        D_name = self.D1.detector_combox.currentText()
        #D_exp = self.D1.detector_now_lineedit.text()
        #for i in range(2):
        M1_name = self.M1.get_M_name()    
        #M1_name = self.M1.motor_pv_combox.currentText()
        M1_start,M1_end,M1_num,D_exp = self.M1.scan_para()
        #M2_name = self.M2.motor_pv_combox.currentText()
        M2_name = self.M2.get_M_name()  
        M2_start,M2_end,M2_num = self.M2.scan_para()
        self.trigger_twoD_My_step.emit(self.M1.table_motor.item(0,2).text())
        self.mrc_mnc.twoD_scan_start(M1_name,M1_start,M1_end,M1_num,M2_name,M2_start,M2_end,M2_num,D_name,D_exp)
        self.btn_scan_start.setEnabled(False)
        self.btn_scan_end.setEnabled(True)
        #self.btn_scan_pause.setEnabled(True)

        
    def scan_end(self):
        #print("&&&&&&&&&")
        self.btn_scan_start.setEnabled(True)
        #self.btn_scan_end.setEnabled(False)
        #self.btn_scan_pause.setEnabled(False)
        self.mrc_mnc.twoD_scan_end()
        self.trigger_twoD_scan_status.emit('twoD_scan_end')
    def scan_pause(self):
        self.scan_paused = True
        #self.btn_scan_start.setEnabled(True)
        #self.btn_scan_end.setEnabled(False)
        #self.btn_scan_pause.setEnabled(False)
        self.mrc_mnc.twoD_scan_pause()
        self.trigger_twoD_scan_status.emit('twoD_scan_pause')
    def scan_resume(self):
        self.scan_paused = False
        #self.btn_scan_start.setEnabled(False)
        #self.btn_scan_end.setEnabled(True)
        #self.btn_scan_pause.setEnabled(True)
        self.mrc_mnc.twoD_scan_resume()
        self.trigger_twoD_scan_status.emit('twoD_scan_resume')

    def start_count(self,scantype):
        #print(scantype)
        if self.count_paused:
            self.resume_count()
        else:
            try:
               self.img_widget.clear_show()
            except:
               pass
            try:
               self.xbpm_widget.clear_show()
            except:
               pass
            #self.img_widget.clear_show()
            #self.xbpm_widget.clear_show()
            D_time = self.D1.detector_now_lineedit.text()
            D_name = self.D1.detector_combox.currentText()
            D_loop = self.D1.loop_lineedit.text()
            self.mrc_mnc.D_start(D_name, D_time, D_loop, str(scantype))
    def end_count(self):
        self.mrc_mnc.D_end()
        D_name = self.D1.detector_combox.currentText()
        if D_name in self.detector_image:
            #cmd_monitor =  "%go "+ D_name +".cam.acquire.set(%d).wait()\n" % int(1)
            #self.mrc.req_rep_base("cmd", go = "", cmd = cmd_monitor)
            #self.mrc_mnc.do_cmd(cmd_monitor)
            self.mrc_mnc.D_image_cam(D_name)
        else:
            pass
        #self.D1.btn_D_start.setEnabled(True)
        #self.D1.btn_D_pause.setEnabled(False)
        #self.D1.btn_D_end.setEnabled(False)
        self.btn_scan_start.setEnabled(True)
        #self.btn_scan_end.setEnabled(False)
        #self.btn_scan_pause.setEnabled(False)
    def pause_count(self):
        self.count_paused = True
        self.mrc_mnc.D_pause()
        #self.D1.btn_D_start.setEnabled(True)
        #self.D1.btn_D_pause.setEnabled(False)
        #self.D1.btn_D_end.setEnabled(False)

    def resume_count(self):
        self.count_paused = False
        self.mrc_mnc.D_resume()
        #self.D1.btn_D_start.setEnabled(True)
        #self.D1.btn_D_pause.setEnabled(False)
        #self.D1.btn_D_end.setEnabled(False)





if __name__ == '__main__':        
    app = QApplication(sys.argv)
    w = two_image_scan()
    w.show()
    sys.exit(app.exec_())



from PyQt5.QtWidgets import QWidget,QHBoxLayout,QTableWidget,QPushButton,QApplication,QVBoxLayout,QTableWidgetItem,QCheckBox,QAbstractItemView,QHeaderView,QLabel,QFrame,QLineEdit,QComboBox,QGridLayout,QGroupBox,QMainWindow,QRadioButton, QToolBar
from PyQt5 import QtWidgets,QtCore
from PyQt5.QtCore import pyqtSignal, QPoint, Qt, QSize, QRectF, QPointF
from PyQt5.QtGui import  QFont,QColor, QPixmapCache, QIcon, QPen
#from faker import Factory
import random
import sys
import math
import pyqtgraph as pg
from pyqtgraph import GraphicsLayoutWidget, mkPen, mkBrush
from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph.graphicsItems.ViewBox import ViewBox
from collections import defaultdict
from pyqtgraph import Point, GraphicsObject
from numpy import errstate,isneginf,array
from qt_material import apply_stylesheet
import qdarkstyle
import numpy as np
from .CurvePlot_widget import CurvePlot
#from imageshow_widget import image_show
from .QWidget_widget import label_widget, button_widget
import datetime

class xbpm_widget(QWidget):
    trigger_xbpm_k_b = pyqtSignal(float,float,float,float)
    def __init__(self,n):
        super().__init__()
        pg.setConfigOption('background', 'w')
        pg.setConfigOption('foreground', 'k')
        pg.setConfigOptions(imageAxisOrder='row-major')
        #screen_size = desktop.screenGeometry()
        #width = screen_size.width()
        #height = screen_size.height()
        #self.width = int(width/2)
        #self.height = int(height*0.8)
        self.y_index = 0
        self.y_axis = None
        self.y_axis_get = False
        self.save_path = '/home/mamba/FS_tif'
        self.color_list = [(200,0,0),(0,150,0),(0,0,200),(0,0,0)]
        self.y_axis_name = ['sum_I','I','Ia+Ib & Ic+Id','Ia+Ic & Ib+Id','Ia+Ib/(Ic+Id)','Ia+Ic/(Ib+Id)']
        self.plot_items = []
        self.infs = []
        #self.vbox_plt = QVBoxLayout()
        self.vbox_plt = QGridLayout()
        self.plotimg_item = pg.PlotItem()
        #self.plotview = pg.PlotWidget(background="w", viewBox=InteractiveViewBoxC(self))
        ##self.plotview.setXRange(0,2000)
        #self.plotview.getPlotItem().addLegend()
        #self.plot = self.plotview.getPlotItem()
        #self.plot.hideButtons()  # hide the autorange button
        #self.plot.setDownsampling(auto=True, mode="peak")
        self.plot_item = CurvePlot()
        self.plot_item.set_axis_name("M",self.y_axis_name[1])
        #self.plot_item.setFixedSize(500, 200)
        self.plot_items.append(self.plot_item)
        self.vbox_plt.addWidget(self.plot_item,0,0)
        #for i in range(math.ceil(n/2)):
        #self.n_index = n
        #for i in range(math.ceil(n/2)):
        #    for j in range(2):
        #        self.plot_item = CurvePlot()
        #        self.plot_item.set_axis_name("M",self.y_axis_name[2*i+j])
        #        #self.plot_item.setFixedSize(500, 200)
        #        self.plot_items.append(self.plot_item)
        #        self.vbox_plt.addWidget(self.plot_item,i,j)
        #for i in range(4):
        #     inf = pg.InfiniteLine(movable=True, angle=90, pen=pg.mkPen(color='#FFA500',width = 3),label='x={value:0.2f}', labelOpts={'position':0.1, 'color':'#FF0000', 'fill': (200,200,200,200), 'movable': True})
        #     inf.setPos([0.5*i, 0.5*i])
        #     self.infs.append(inf)
        #self.plot_items[4].plot.addItem(self.infs[0])
        #self.plot_items[4].plot.addItem(self.infs[1])
        #self.plot_items[5].plot.addItem(self.infs[2])
        #self.plot_items[5].plot.addItem(self.infs[3])
        #self.legend1 = self.plot_items[1].addLegend(frame=True, colCount=1, offset=(-10, 5))
        self.hbox = QHBoxLayout()
        #vbox.setContentsMargins(0, 0, 0, 0)
        #vbox.addWidget(self.navbar)
        self.hbox.addLayout(self.vbox_plt,1)
        #self.hbox.addWidget(self.image_view,1)
        
        #self.setLayout(vbox)
        groupbox1 = QGroupBox("")
        groupbox1.setLayout(self.hbox)
        twogroup = QHBoxLayout()
        twogroup.addWidget(groupbox1)
        self.setLayout(twogroup)
        #self.show()
    def update_2d_motor_detector(self,motor_y,motor_x,detector):
        self.name_detector = detector
        self.name_motor_x = motor_x
        self.name_motor_y = motor_y
        #print(self.name_motor_x,self.name_detector)
    def update_motor_detector(self,motor,detector):
        self.name_detector = detector
        self.name_motor_x = motor
        #print(self.name_motor_x,self.name_detector)
    def update_plt_data(self,data_xbpm):
        sum_xbpm = self.get_sum_xbpm(data_xbpm)
        sum_x_up,sum_x_down,up_x_down = self.sum_up_xbpm(data_xbpm)
        sum_x_left,sum_x_right,left_x_right = self.sum_left_xbpm(data_xbpm)
        self.sum_xbpm.append(sum_xbpm)
        self.sum_x_up.append(sum_x_up)
        self.sum_x_down.append(sum_x_down)
        self.up_x_down.append(up_x_down)
        self.sum_x_left.append(sum_x_left)
        self.sum_x_right.append(sum_x_right)
        self.left_x_right.append(left_x_right)
        self.xbpm_A.append(data_xbpm[0])
        self.xbpm_B.append(data_xbpm[1])
        self.xbpm_C.append(data_xbpm[2])
        self.xbpm_D.append(data_xbpm[3])

        self.tot_sum_xbpm.append(sum_xbpm)
        self.tot_sum_x_up.append(sum_x_up)
        self.tot_sum_x_down.append(sum_x_down)
        self.tot_up_x_down.append(up_x_down)
        self.tot_sum_x_left.append(sum_x_left)
        self.tot_sum_x_right.append(sum_x_right)
        self.tot_left_x_right.append(left_x_right)
        self.tot_xbpm_A.append(data_xbpm[0])
        self.tot_xbpm_B.append(data_xbpm[1])
        self.tot_xbpm_C.append(data_xbpm[2])
        self.tot_xbpm_D.append(data_xbpm[3])

        
        if self.x_M:
           self.x_axis = list(range(len(self.sum_xbpm)))
           self.tot_x_axis = list(range(len(self.tot_sum_xbpm)))
        self.update_2d_image()
        tot_data = [self.tot_sum_xbpm,self.tot_up_x_down,self.tot_left_x_right]
        if self.y_axis_get:
        #for i in range(len(tot_data)):
            a,b = self.get_max_min(self.tot_x_axis,tot_data[0])
            c,d = self.get_max_min(self.tot_y_axis,tot_data[0])
            labels = 'max= '+'('+str(a[0])+','+str(c[0])+','+str(a[1])+')'+', min ='+'('+str(b[0])+','+str(d[0])+','+str(b[1])+')'
            #labels = 'max= '+str(a)+', min ='+str(b)
            self.plot_items[0].max_min_label.setText(labels)
            a,b = self.get_max_min(self.tot_x_axis,tot_data[1])
            c,d = self.get_max_min(self.tot_y_axis,tot_data[1])
            labels = 'max= '+'('+str(a[0])+','+str(c[0])+','+str(a[1])+')'+', min ='+'('+str(b[0])+','+str(d[0])+','+str(b[1])+')'
            #labels = 'max= '+str(a)+', min ='+str(b)
            self.plot_items[4].max_min_label.setText(labels)
            a,b = self.get_max_min(self.tot_x_axis,tot_data[2])
            c,d = self.get_max_min(self.tot_y_axis,tot_data[2])
            labels = 'max= '+'('+str(a[0])+','+str(c[0])+','+str(a[1])+')'+', min ='+'('+str(b[0])+','+str(d[0])+','+str(b[1])+')'
            #labels = 'max= '+str(a)+', min ='+str(b)
            self.plot_items[5].max_min_label.setText(labels)
        else:
            a,b = self.get_max_min(self.tot_x_axis,tot_data[0])
            labels = 'max= '+str(a)+', min ='+str(b)
            self.plot_items[0].max_min_label.setText(labels)
            a,b = self.get_max_min(self.tot_x_axis,tot_data[1])
            labels = 'max= '+str(a)+', min ='+str(b)
            self.plot_items[4].max_min_label.setText(labels)
            a,b = self.get_max_min(self.tot_x_axis,tot_data[2])
            labels = 'max= '+str(a)+', min ='+str(b)
            self.plot_items[5].max_min_label.setText(labels)
    def update_step_y(self,step_y):
        self.step_y = float(step_y)
    def update_plt_x(self,x):
        self.x_M = False
        self.x_axis.append(x)
        self.tot_x_axis.append(x)
    def update_2d_image(self):
        if (len(self.y_axis)>=2):
           if ((self.y_axis[-1]-self.y_axis[-2])>= 0.5*self.step_y):
                self.y_index = +1
                #self.sum_xbpm = [self.sum_xbpm[-1]]
                #self.sum_x_up = [self.sum_x_up[-1]]
                #self.sum_x_down = [self.sum_x_down[-1]]
                #self.up_x_down = [self.up_x_down[-1]]
                #self.sum_x_left = [self.sum_x_left[-1]]
                #self.sum_x_right = [self.sum_x_right[-1]]
                #self.left_x_right = [self.left_x_right[-1]]
                self.x_axis = [self.x_axis[-1]]
                #self.y_axis = []
                self.xbpm_A = [self.xbpm_A[-1]]
                #self.xbpm_B = [self.xbpm_B[-1]]
                #self.xbpm_C = [self.xbpm_C[-1]]
                #self.xbpm_D = [self.xbpm_D[-1]]
                self.update_multi_plot(self.xbpm_A,self.x_axis)
           else:
                self.update_multi_plot(self.xbpm_A,self.x_axis)
              
        else:
           self.plot_items[1].plot.clear()
           self.plot_items[2].plot.clear()
           self.plot_items[3].plot.clear()
           self.update_plot(self.xbpm_A,self.x_axis)
    def update_plt_y(self,y):
        self.y_axis_get = True
        #print(y)
        self.y_axis.append(y)
        self.tot_y_axis.append(y)
    def update_multi_plot(self,xbpm_A,x):#xbpm_B,xbpm_C,xbpm_D, sum_xbpm, sum_x_up, sum_x_down, up_x_down, sum_x_left, sum_x_right, left_x_right, x): 
        #self.plot_items[0].plot.plot(x,sum_xbpm,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        self.plot_items[0].add_curve(1,"I")
        self.plot_items[0].set_values(1,x,xbpm_A)
        #self.plot_items[1].plot.plot(x,xbpm_A,pen=pg.mkPen(color=self.color_list[0],width=2),brush=pg.mkBrush(color=self.color_list[0]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[0]),symbolBrush=pg.mkBrush(color=self.color_list[0]))
        #self.plot_items[1].plot.plot(x,xbpm_B,pen=pg.mkPen(color=self.color_list[1],width=2),brush=pg.mkBrush(color=self.color_list[1]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[1]),symbolBrush=pg.mkBrush(color=self.color_list[1]))
        #self.plot_items[1].plot.plot(x,xbpm_C,pen=pg.mkPen(color=self.color_list[2],width=2),brush=pg.mkBrush(color=self.color_list[2]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[2]),symbolBrush=pg.mkBrush(color=self.color_list[2]))
        #self.plot_items[1].plot.plot(x,xbpm_D,pen=pg.mkPen(color=self.color_list[3],width=2),brush=pg.mkBrush(color=self.color_list[3]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[3]),symbolBrush=pg.mkBrush(color=self.color_list[3]))
        #
        #self.plot_items[2].plot.plot(x,sum_x_up,pen=pg.mkPen(color=self.color_list[0],width=2),brush=pg.mkBrush(color=self.color_list[0]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[0]),symbolBrush=pg.mkBrush(color=self.color_list[0]))
        #self.plot_items[2].plot.plot(x,sum_x_down,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        #self.plot_items[4].plot.plot(x,up_x_down,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        #self.plot_items[3].plot.plot(x,sum_x_left,pen=pg.mkPen(color=self.color_list[0],width=2),brush=pg.mkBrush(color=self.color_list[0]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[0]),symbolBrush=pg.mkBrush(color=self.color_list[0]))
        #self.plot_items[3].plot.plot(x,sum_x_right,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        #self.plot_items[5].plot.plot(x,left_x_right,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        self.save_tif()
        
    def update_plot(self,xbpm_A,x):#bpm_B,xbpm_C,xbpm_D, sum_xbpm, sum_x_up, sum_x_down, up_x_down, sum_x_left, sum_x_right, left_x_right, x): 
        self.plot_items[0].add_curve(1,"I")
        self.plot_items[0].set_values(1,x,xbpm_A)
        #self.plot_items[0].plot.plot(x,sum_xbpm,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        #self.plot_items[1].plot.plot(x,xbpm_A,pen=pg.mkPen(color=self.color_list[0],width=2),brush=pg.mkBrush(color=self.color_list[0]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[0]),symbolBrush=pg.mkBrush(color=self.color_list[0]),name='I_a')
        #self.plot_items[1].plot.plot(x,xbpm_B,pen=pg.mkPen(color=self.color_list[1],width=2),brush=pg.mkBrush(color=self.color_list[1]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[1]),symbolBrush=pg.mkBrush(color=self.color_list[1]),name='I_b')
        #self.plot_items[1].plot.plot(x,xbpm_C,pen=pg.mkPen(color=self.color_list[2],width=2),brush=pg.mkBrush(color=self.color_list[2]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[2]),symbolBrush=pg.mkBrush(color=self.color_list[2]),name='I_c')
        #self.plot_items[1].plot.plot(x,xbpm_D,pen=pg.mkPen(color=self.color_list[3],width=2),brush=pg.mkBrush(color=self.color_list[3]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[3]),symbolBrush=pg.mkBrush(color=self.color_list[3]),name='I_d')
        #
        #self.plot_items[2].plot.plot(x,sum_x_up,pen=pg.mkPen(color=self.color_list[0],width=2),brush=pg.mkBrush(color=self.color_list[0]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[0]),symbolBrush=pg.mkBrush(color=self.color_list[0]),name='Ia+Ib')
        #self.plot_items[2].plot.plot(x,sum_x_down,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)),name='Ic+Id')
        #self.plot_items[4].plot.plot(x,up_x_down,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        #self.plot_items[3].plot.plot(x,sum_x_left,pen=pg.mkPen(color=self.color_list[0],width=2),brush=pg.mkBrush(color=self.color_list[0]),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=self.color_list[0]),symbolBrush=pg.mkBrush(color=self.color_list[0]),name='Ia+Ic')
        #self.plot_items[3].plot.plot(x,sum_x_right,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)),name='Ib+Id')
        #self.plot_items[5].plot.plot(x,left_x_right,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        self.save_tif()
        #self.plot_items[2].plot.plot(x,cent_y_pos,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        #self.plot_items[3].plot.plot(x,cent_pos,pen=pg.mkPen(color=(0,0,0),width=2),brush=pg.mkBrush(color=(0,0,0)),symbol='o',symbolSize=7,symbolPen=pg.mkPen(color=(0,0,0)),symbolBrush=pg.mkBrush(color=(0,0,0)))
        #self.plot_items[1].plot.legend.addItem(self.plot_items[1].plot.curves[0],'I_a')
#    def find_nearest(self,array_x,array_y,value):
#        array_x = np.asarray(array_x)
#        array_y = np.asarray(array_y)
#        idx = (np.abs(array_x-value).argmin())
#        return array_x[idx],array_y[idx]
#    def init_k_b(self,k_up,b_up,k_left,b_left):
#        self.k_up = k_up
#        self.b_up = b_up
#        self.k_left = k_left
#        self.b_left = b_left
 #   def xbpm_cali(self):
 #       lin1 = self.infs[0].value()
 #       lin2 = self.infs[1].value()
 #       Mup_low = np.min([lin1,lin2])
 #       Mup_high = np.max([lin1,lin2])
 #       m1,y1 =  self.find_nearest(self.x_axis,self.up_x_down, Mup_low)
 #       m2,y2 =  self.find_nearest(self.x_axis,self.up_x_down, Mup_high)
 #       self.k_up = (y2-y1)/(m2-m1)
 #       self.b_up = y2-self.k_up*m2
 #
 #       lin3 = self.infs[2].value()
 #       lin4 = self.infs[3].value()
 #       Mleft_low = np.min([lin3,lin4])
 #       Mleft_high = np.max([lin3,lin4])
 #       m3,y3 =  self.find_nearest(self.x_axis,self.up_x_down, Mleft_low)
 #       m4,y4 =  self.find_nearest(self.x_axis,self.up_x_down, Mleft_high)
 #       self.k_left = (y4-y3)/(m4-m3)
 #       self.b_left = y4-self.k_left*m4
 #       self.trigger_xbpm_k_b.emit(self.k_up,self.b_up,self.k_left,self.b_left)
 #   def get_sum_xbpm(self,data_xbpm):
 #       sum_xbpm = np.sum(data_xbpm)
 #       return(sum_xbpm)
 #   def sum_left_xbpm(self,data_xbpm):
 #       m = data_xbpm[0]+data_xbpm[2]
 #       n = data_xbpm[1]+data_xbpm[3]
 #       sum_dev = self.k_left*((m-n)/(m+n))+self.b_left
 #       return(m,n,sum_dev)
 #   def sum_up_xbpm(self,data_xbpm):
 #       m = data_xbpm[0]+data_xbpm[1]
 #       n = data_xbpm[2]+data_xbpm[3]
 #       sum_dev = self.k_up*((m-n)/(m+n))+self.b_up
 #       return(m,n,sum_dev)
    def clear_show(self):
        self.x_M = True
        self.y_axis_get = False
        #self.sum_xbpm = []
        #self.sum_x_up = []
        #self.sum_x_down = []
        #self.up_x_down = []
        #self.sum_x_left = []
        #self.sum_x_right = []
        #self.left_x_right = []
        self.x_axis = []
        self.y_axis = []
        self.xbpm_A = []
        #self.xbpm_B = []
        #self.xbpm_C = []
        #self.xbpm_D = []
        #self.tot_sum_xbpm = []
        #self.tot_sum_x_up = []
        #self.tot_sum_x_down = []
        #self.tot_up_x_down = []
        #self.tot_sum_x_left = []
        #self.tot_sum_x_right = []
        #self.tot_left_x_right = []
        self.tot_x_axis = []
        self.tot_y_axis = []
        self.tot_xbpm_A = []
        #self.tot_xbpm_B = []
        #self.tot_xbpm_C = []
        #self.tot_xbpm_D = []
        #for i in range((math.ceil(self.n_index/2))*2):
        self.plot_items[0].plot.clear()

    def get_max_min(self,list_x,list_y):
        max_a = list_y.index(max(list_y))
        min_a = list_y.index(min(list_y))
        #print(max_a)
        
        max_pos = (float("%.8f" % float(list_x[max_a])), float( "%.8f" % list_y[max_a]))
        min_pos = (float("%.8f" %list_x[min_a]), float( "%.8f" % list_y[min_a]))
        #print(max_pos,min_pos)
        return(max_pos,min_pos)            
    def save_tif(self):
        time_now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        save_path = self.save_path
        try:
              with open("/home/mamba/FS_tif/center_plt.txt","a") as f:
                  if self.y_axis_get:
                      f.writelines(self.name_motor_x +" "+"%13.8f" % self.x_axis[-1]+" "+self.name_motor_y+" "+"%13.8f" % self.y_axis[-1]+" "+ self.name_detector+" "+ "%13.8f" % self.xbpm_A[-1]+" "+time_now)
                  else:
              
                      f.writelines(self.name_motor_x+" "+"%13.8f" % self.x_axis[-1]+" "+self.name_detector+" "+"%13.8f" % self.xbpm_A[-1]+" "+time_now) 
                  
                  f.writelines("\n")
        except:
              self.name_motor_x = 'count'
              self.name_detector = 'XBPM'
              with open("/home/mamba/FS_tif/center_plt.txt","a") as f:
                  if self.y_axis_get:
                      f.writelines(self.name_motor_x +" "+"%13.8f" % self.x_axis[-1]+" "+self.name_motor_y+" "+"%13.8f" % self.y_axis[-1]+" "+ self.name_detector+" "+ "%13.8f" % self.xbpm_A[-1]+" "+"%13.8f" % self.xbpm_B[-1]+" "+"%13.8f" % self.xbpm_C[-1]+" "+"%13.8f" % self.xbpm_D[-1]+" "+"%13.8f" % self.sum_xbpm[-1]+" "+"%13.8f" % self.sum_x_up[-1]+" "+"%13.8f" % self.sum_x_down[-1]+" "+"%13.8f" % self.up_x_down[-1]+" "+"%13.8f" % self.sum_x_left[-1]+" "+"%13.8f" % self.sum_x_right[-1]+" "+"%13.8f" % self.left_x_right[-1]+" "+time_now)
                  else:
                      f.writelines(self.name_motor_x+" "+"%13.8f" % self.x_axis[-1]+" "+self.name_detector+" "+"%13.8f" % self.xbpm_A[-1]+" "+"%13.8f" % self.xbpm_B[-1]+" "+"%13.8f" % self.xbpm_C[-1]+" "+"%13.8f" % self.xbpm_D[-1]+" "+"%13.8f" % self.sum_xbpm[-1]+" "+"%13.8f" % self.sum_x_up[-1]+" "+"%13.8f" % self.sum_x_down[-1]+" "+"%13.8f" % self.up_x_down[-1]+" "+"%13.8f" % self.sum_x_left[-1]+" "+"%13.8f" % self.sum_x_right[-1]+" "+"%13.8f" % self.left_x_right[-1]+" "+time_now) 
                  
                  f.writelines("\n")
     
if __name__ == '__main__':
    app = QApplication(sys.argv)
    desktop = app.desktop()
    w = xbpm_widget(3)
    w.show()
    sys.exit(app.exec_())



import sys
from PyQt5.QtWidgets import QWidget,QHBoxLayout,QTableWidget,QPushButton,QApplication,QVBoxLayout,QTableWidgetItem,QCheckBox,QAbstractItemView,QHeaderView,QLabel,QFrame,QLineEdit,QComboBox,QGridLayout,QGroupBox,QMainWindow,QRadioButton, QToolBar
from PyQt5 import QtWidgets,QtCore
from PyQt5.QtCore import pyqtSignal, QPoint, Qt, QSize, QRectF, QPointF
from PyQt5.QtGui import  QFont,QColor, QPixmapCache, QIcon, QPen
import pyqtgraph as pg
from pyqtgraph import GraphicsLayoutWidget, mkPen, mkBrush
from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph.graphicsItems.ViewBox import ViewBox
from collections import defaultdict
from pyqtgraph import Point, GraphicsObject
from numpy import errstate,isneginf,array
from .QWidget_widget import InteractiveViewBoxC






class CurvePlot(QWidget):
    limits_changed = pyqtSignal()

    def __init__(self, parent=None):
        super(CurvePlot, self).__init__(parent)
        self.setContentsMargins(0, 0, 0, 0)
        self.plotview = pg.PlotWidget(background="w", viewBox=InteractiveViewBoxC(self))
        #self.plotview.setXRange(0,2000)
        self.plotview.getPlotItem().addLegend()
        self.plot = self.plotview.getPlotItem()
        self.plot.hideButtons()  # hide the autorange button
        self.plot.setDownsampling(auto=True, mode="peak")
        self._curves = {}
        self._current_vline = None

        self.markings = []
        self.vLine = pg.InfiniteLine(angle=90, movable=False)
        self.hLine = pg.InfiniteLine(angle=0, movable=False)
        self.plot.scene().sigMouseMoved.connect(self.mouse_moved_viewhelpers)
        self.plot.scene().sigMouseMoved.connect(self.plot.vb.mouseMovedEvent)
        self.plot.setLabel('bottom', 'M')
        # interface settings
        self.markclosest = True  # mark
        self.crosshair = True

        # ----------------QToolBar---------------------------------------
        self.navbar = QToolBar()
        self.navbar.setIconSize(QSize(20, 20))
        self.navbar.setStyleSheet("font-size:15px;")
        self.navbar.addAction(QIcon('/home/mamba/mamba/mamba/frontend/icon/button_grid.png'),
                              "Show grid", self.grid_changed)
        self.navbar.addAction(QIcon('/home/mamba/mamba/mamba/frontend/icon/button_big.png'),
                              "Zoom in", self.plot.vb.set_mode_zooming)
        self.navbar.addAction(QIcon('/home/mamba/mamba/mamba/frontend/icon/button_arrow.png'),
                              "move", self.plot.vb.set_mode_panning)
        self.navbar.addAction(QIcon('/home/mamba/mamba/mamba/frontend/icon/button_fount.png'),
                              "auto", self.plot.vb.enableAutoRangeTrue)
        #self.navbar.addAction(QIcon('/home/mamba/mamba/mamba/frontend/icon/log.png'),
         #                     "log", self.set_log)

        spacer1 = QWidget()
        spacer2 = QWidget()
        #spacer2.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.navbar.addWidget(spacer1)
        self.axis_label = QLabel("")
        self.max_min_label = QLabel("")
        self.navbar.addWidget(self.axis_label)
        self.navbar.addWidget(spacer2)
        self.navbar.addWidget(self.max_min_label)
        vbox = QVBoxLayout()
        #vbox.setContentsMargins(0, 0, 0, 0)
        vbox.addWidget(self.navbar)
        vbox.addWidget(self.plotview)
        #self.setLayout(vbox)
        groupbox1 = QGroupBox("")
        groupbox1.setLayout(vbox)
        twogroup = QHBoxLayout()
        twogroup.addWidget(groupbox1)
        self.setLayout(twogroup)
        # -------------------------------------------------------------------

        self.show_grid = False
        self.log = False
        self.grid_changed()

    def grid_changed(self):
        self.show_grid = not self.show_grid
        self.grid_apply()

    def set_log(self):
        self.log = not self.log
        for curve_id in self._curves:
            if self._curves[curve_id]["y"] is not None:
               self.set_values(curve_id, self._curves[curve_id]["x"], self._curves[curve_id]["y"])

    def grid_apply(self):
        self.plot.showGrid(self.show_grid, self.show_grid)

    def mouse_moved_viewhelpers(self, pos):
        if self.plot.sceneBoundingRect().contains(pos):
            mousePoint = self.plot.vb.mapSceneToView(pos)
            posx, posy = mousePoint.x(), mousePoint.y()
            labels = str(round(posx, 4)) + "   " + str(round(posy, 4))
            self.axis_label.setText(labels)

    def add_curve(self, curve_id, curve_name, curve_color=QColor('blue'), markers_on=False):
        pen = mkPen(curve_color, width=2)
        symbol = "o"
        symbolPen = mkPen(QColor('red'))
        symbolBrush = mkBrush(curve_color)
        # this adds the item to the plot and legend
        if markers_on:
            a = self.plot
            plot = self.plot.plot(name=curve_name, pen=pen, symbol=symbol, symbolPen=symbolPen,
                                  symbolBrush=symbolBrush, symbolSize=4)
            # a.setLabel('left', "Y Axis", units='A')
            # a.setLabel('bottom', "Y Axis", units='s')
        else:
            a = self.plot
            plot = a.plot(name=curve_name, pen=pen)

            # a.setLabel('left', "Y Axis", units='A')
            # a.setLabel('bottom', "Y Axis", units='s')
        self._curves[curve_id] = {}
        self._curves[curve_id]["plot"] = plot
        self._curves[curve_id]["x"] = None
        self._curves[curve_id]["y"] = None

    def remove_curve(self, curve_id):
        if curve_id in self._curves:
            self.plot.removeItem(self._curves[curve_id]["plot"])
            del self._curves[curve_id]
            self._update_legend()

    def _update_legend(self):
        # clear and rebuild legend (there is no remove item method for the legend...)
        self.plot.clear()
        self.plot.getPlotItem().legend.items = []
        for curve in self._curves.values():
            self.plot.addItem(curve["plot"])
        if self._current_vline:
            self.plot.addItem(self._current_vline)

    def set_values(self, curve_id, data_x, data_y):
        self._curves[curve_id]["x"] = data_x
        self._curves[curve_id]["y"] = data_y
        curve = self._curves[curve_id]["plot"]
        if self.log:
            with errstate(divide='ignore'):
                data_y = np.log10(data_y)
                data_y[isneginf(data_y)] = 0
        curve.setData(data_x, data_y)

    def vline(self, x, color):
        if self._current_vline:
            self.plot.removeItem(self._current_vline)
        self._current_vline = self.plot.addLine(x=x, pen=color)
        # self._current_vline.sigDragged.connect(self.aaa)
    
    def set_axis_name(self, x_name=None, y_name=None):
        if x_name is not None:
            self.plot.setLabel('bottom', x_name)
        if y_name is not None:
            self.plot.setLabel('left', y_name)

if __name__ == '__main__':
    #app = QApplication(sys.argv)
    #w = MainWindow()
    #sys.exit(app.exec_())
    app=QApplication(sys.argv)
    demo = CurvePlot()
    demo.show()
    #app.setStyleSheet(qdarkstyle.load_stylesheet(palette=LightPalette()))
    sys.exit(app.exec_())



#!/usr/bin/python3

#from distutils.core import setup
from setuptools import setup, find_packages

with open("README.rst", "r") as f:
  long_description = f.read()

setup(

    name="dim_relayout",
    version="0.0.5",
    #packages= ["dim_relayout"],
    author="Yingke Huang",
    author_email="ykhuang@ihep.ac.cn",
    description="dim relayout",
    install_requires=[],	# 依赖包会同时被安装
    license='GPL',
    packages=find_packages('src/')
)



#!/usr/bin/python3
from PyQt5.QtWidgets import QWidget,QHBoxLayout,QTableWidget,QPushButton,QApplication,QVBoxLayout,QTableWidgetItem,QCheckBox,QAbstractItemView,QHeaderView,QLabel,QFrame,QLineEdit,QComboBox,QGridLayout,QGroupBox,QMainWindow,QRadioButton, QToolBar
from PyQt5 import QtWidgets,QtCore
from PyQt5.QtCore import pyqtSignal, QPoint, Qt, QSize, QRectF, QPointF
from PyQt5.QtGui import  QFont,QColor, QPixmapCache, QIcon, QPen
#from faker import Factory
import random
import sys
import time
import pyqtgraph as pg
from pyqtgraph import GraphicsLayoutWidget, mkPen, mkBrush
from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph.graphicsItems.ViewBox import ViewBox
from collections import defaultdict
from pyqtgraph import Point, GraphicsObject
from numpy import errstate,isneginf,array
from qt_material import apply_stylesheet
import qdarkstyle
from qdarkstyle.light.palette import LightPalette
from .D_widget import D_widget
from .M_widget import M_widget
from .CurvePlot_widget import CurvePlot
from .imageshow_widget import image_show
from .xbpm_widget import xbpm_widget
from .QWidget_widget import label_widget, button_widget
from .mrc_mnc_backend import mrc_mnc
#sys.setrecursionlimit(2000)
class save_fil(QWidget):
    update_path_signal = pyqtSignal(object)
   
    def __init__(self,path):
        super().__init__()
        self.btn_save = QtWidgets.QPushButton("Save")
        self.btn_save.setFixedSize(70,70)
        self.lineedit_save = QtWidgets.QLineEdit(path)
        self.lineedit_save.setFixedSize(250,70)
        #self.lineedit_name = QtWidgets.QLineEdit(path)
        #self.lineedit_name.setFixedSize(150,70)
        #self.lineedit_save.setReadOnly(True)
        self.lineedit_save.setEnabled(False)
        self.hlayout = QtWidgets.QHBoxLayout()
        #self.hlayout.addWidget(self.label_save)
        self.hlayout.addWidget(self.lineedit_save)
        #self.hlayout.addWidget(self.lineedit_name)
        self.hlayout.addWidget(self.btn_save)
        self.setLayout(self.hlayout)
        self.btn_save.clicked.connect(self.fil_save)
    def fil_save(self):
        options = QtWidgets.QFileDialog.Options()
        options |=  QtWidgets.QFileDialog.DontUseNativeDialog
        directory = QtWidgets.QFileDialog.getExistingDirectory(None,'open file dialog','/home/mamba/FS_tif',options=options)
        self.lineedit_save.setText(directory)
        self.update_path_signal.emit(directory)

class one_image_scan(QWidget):
    trigger_oneD_scan_status = pyqtSignal(str)
    #trigger_twoD_scan_status = pyqtSignal(str) 
    def __init__(self,mrc,mnc):
        super().__init__()
        #self.table_motor = Table_widget()
        self.msg = None
        self.plt_widget = CurvePlot()
        self.img_widget = image_show(4)
        self.xbpm_widget = xbpm_widget(6)
        self.mrc_mnc = mrc_mnc(mrc,mnc)
        self.scan_paused = False
        self.count_paused = False
        self.save_path = save_fil("/home/mamba/FS_tif")
        font = QFont('Times New Roman',15)
        #self.motor_list = {'slit1': ['M1.x','M1.y','M1.z'],'slit2':['M2.x','M2.y','M2.z'],'slit3':['M3.x','M3.y','M3.z']}
        self.scan_label = QLabel("scan type:    ")
        self.scan_label.setFont(font)
        self.scan_label.setFixedSize(150,40)
        #self.detector_image = ['荧光靶','camera']
        #self.detector_point = ['电离室']
        #self.detector_xbpm = ['XBPM']
        self.detector_image = ['D.WhiteFS','D.IntegratedFS', 'D.MonochromaticFS', 'D.HRFS1', 'D.HRFS2']
        self.detector_point = ['电离室']
        self.detector_xbpm = ['D.WhiteXBPM','D.QXBPM']
        
        #self.motor_label = label_widget("设备")
        #self.motor_combox = QComboBox()
        #self.detector_combox = QComboBox(self)
        #self.motor_pv_combox = QComboBox(self)
        #self.motor_combox.setFixedSize(100, 40)
        #self.motor_pv_combox.setFixedSize(100, 40)
        #self.detector_combox.setFixedSize(100, 40)
        self.one_Dscan = QRadioButton("1D scan")
        self.two_Dscan = QRadioButton("2D scan")
        self.one_Dscan.setFixedSize(150,40)
        self.one_Dscan.setFont(font)
        self.two_Dscan.setFixedSize(150,40)
        self.two_Dscan.setFont(font)
        self.one_Dscan.setChecked(True)
        #self.motor_vbox = QVBoxLayout()
        #self.motor_vbox.addWidget(self.motor_label)
        #self.motor_vbox.addWidget(self.motor_combox)
        self.scan_type_vbox =  QHBoxLayout()
        self.scan_type_vbox.addWidget(self.scan_label)
        self.scan_type_vbox.addWidget(self.one_Dscan)
        self.scan_type_vbox.addWidget(self.two_Dscan)
        self.btn_scan_start = button_widget('Start scan')
        self.btn_scan_start.setFixedSize(120, 100)
        self.btn_scan_pause = button_widget('Pause scan')
        self.btn_scan_pause.setFixedSize(120, 100)
        self.btn_scan_end = button_widget('End')
        self.btn_scan_end.setFixedSize(120, 100)
        self.hbtnbox = QHBoxLayout()
        self.hbtnbox.addWidget(self.btn_scan_start)
        self.hbtnbox.addWidget(self.btn_scan_pause)
        self.hbtnbox.addWidget(self.btn_scan_end)
        
        self.hbox = QHBoxLayout()
        #self.hbox.addLayout(self.motor_vbox)
        self.hbox.addLayout(self.scan_type_vbox)
        groupbox = QGroupBox("扫描类型")
        font = QFont('Song', 20)
        #groupbox.setFont(font)
        groupbox.setLayout(self.hbox)
        self.scanhbox = QVBoxLayout()
        self.scanhbox.addWidget(groupbox)

        self.M1 = M_widget(1,200,mrc,mnc)
        self.gvbox = QVBoxLayout()
        self.gvbox.addWidget(self.M1)
        self.gvbox.addLayout(self.hbtnbox)
        self.groupbox1 = QGroupBox("电机")
        #self.groupbox1.setFont(font)
        self.groupbox1.setStyleSheet("""QGroupBox::title{font-size:70px;}""")
        self.groupbox1.setLayout(self.gvbox)
        self.D1 = D_widget(1,200,mrc,mnc)

        self.D_cali = button_widget('Calibration')
        self.D_cali.setFixedSize(120, 100)
        self.DBOX = QHBoxLayout()
        self.DBOX.addWidget(self.D1,5)
        self.DBOX.addWidget(self.D_cali,1)   
        #self.motor_combox.addItems(self.motor_list.keys())
        #self.M1.motor_pv_combox.addItems(self.motor_list['slit1'])
        #self.D1.detector_combox.addItems(self.detector_image)
        #self.D1.detector_combox.addItems(self.detector_point)
        #self.D1.detector_combox.addItems(self.detector_xbpm)
        self.group_hbox = QVBoxLayout()
        self.group_hbox.addItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding))
        self.group_hbox.addLayout(self.scanhbox)
        self.group_hbox.addItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding))
        self.group_hbox.addWidget(self.groupbox1)
        self.group_hbox.addItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding))
        #self.group_hbox.addWidget(self.D1)       
        self.group_hbox.addLayout(self.DBOX) 
        self.group_hbox.addWidget(self.save_path)
        self.group_hbox.addItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding))
        self.img_hbox = QHBoxLayout()        
        self.img_hbox.addLayout(self.group_hbox,1)
        self.img_hbox.addWidget(self.img_widget,4)
        self.motor_pv()
        self.M1.motor_combox.currentIndexChanged.connect(self.motor_pv)
        self.M1.motor_pv_combox.currentIndexChanged.connect(self.motor_pv_update)
        self.D1.btn_D_start.clicked.connect(lambda x: self.start_count(str('oneD')))
        self.D1.btn_D_pause.clicked.connect(self.pause_count)
        self.D1.btn_D_end.clicked.connect(self.end_count)
        self.btn_scan_start.clicked.connect(self.scan_start)
        self.btn_scan_end.clicked.connect(self.scan_end)
        self.btn_scan_pause.clicked.connect(self.scan_pause)
      
    def btn_status(self,msg):
        #print("******",msg)
        if msg == 'oneD_scan_stop':
           #self.D1.btn_D_start.setEnabled(True)
           #self.D1.btn_D_pause.setEnabled(False)
           #self.D1.btn_D_end.setEnabled(False)
           self.btn_scan_start.setEnabled(True)
           self.btn_scan_end.setEnabled(False)
           self.btn_scan_pause.setEnabled(False)
        if  msg == 'oneD_scan_start':
           if self.msg == 'oneD_scan_end' or self.msg == 'oneD_scan_pause':
               self.btn_scan_start.setEnabled(True)
               self.btn_scan_end.setEnabled(False)
               self.btn_scan_pause.setEnabled(False)
           else:
               self.btn_scan_start.setEnabled(False)
               self.btn_scan_end.setEnabled(True)
               self.btn_scan_pause.setEnabled(True)
        if  msg == 'oneD_scan_resume':
           self.btn_scan_start.setEnabled(False)
           self.btn_scan_end.setEnabled(True)
           self.btn_scan_pause.setEnabled(True)
        if  msg == 'oneD_scan_end':
           self.btn_scan_start.setEnabled(True)
           self.btn_scan_end.setEnabled(False)
           self.btn_scan_pause.setEnabled(False)
        if  msg == 'oneD_scan_pause':
           self.btn_scan_start.setEnabled(True)
           self.btn_scan_end.setEnabled(False)
           self.btn_scan_pause.setEnabled(False)
        self.msg = msg
    def motor_pv_update(self):
        self.M1.update_pos_now()
        self.M1.init_item()
    def motor_pv(self):
        self.M1.motor_pv_combox.clear()
        motor_now = self.M1.motor_combox.currentText()
        motor_pv = self.M1.motor_list[motor_now]
        self.M1.motor_pv_combox.addItems(motor_pv)
        self.M1.init_item()
        #self.motor_pv_update()
    def scan_start(self):
        if self.scan_paused:
            self.scan_resume()
        else:
            try:
               self.img_widget.clear_show()
            except:
               pass
            try:
               self.xbpm_widget.clear_show()
               self.add_cal_inf()
            except:
               pass
            D_name = self.D1.detector_combox.currentText()
            #D_exp = self.D1.detector_now_lineedit.text()
            #exp_now = self.M1.motor_combox.currentText()
            #M_pv_name = self.M1.motor_pv_combox.currentText()
            M_name = self.M1.get_M_name()
            self.M_current = self.M1.motor_now_lineedit.text()
            M_start,M_end,M_num,D_exp = self.M1.scan_para()
            self.mrc_mnc.oneD_scan_start(M_name,M_start,M_end,M_num,D_name,D_exp)
            self.btn_scan_start.setEnabled(False)
            self.btn_scan_end.setEnabled(True)
    def add_cal_inf(self):
        self.xbpm_widget.plot_items[4].plot.addItem(self.xbpm_widget.infs[0])
        self.xbpm_widget.plot_items[4].plot.addItem(self.xbpm_widget.infs[1])
        self.xbpm_widget.plot_items[5].plot.addItem(self.xbpm_widget.infs[2])
        self.xbpm_widget.plot_items[5].plot.addItem(self.xbpm_widget.infs[3])    
    def scan_end(self):
        #print("&&&&&&&&&")
        self.btn_scan_start.setEnabled(True)
        #self.btn_scan_end.setEnabled(False)
        #self.btn_scan_pause.setEnabled(False)
        self.mrc_mnc.oneD_scan_end()
        self.trigger_oneD_scan_status.emit('oneD_scan_end')
        #time.sleep(1)
        #self.trigger_oneD_scan_status.emit('oneD_scan_end')
        #self.D1.btn_D_start.setEnabled(True)
        #self.D1.btn_D_pause.setEnabled(False)
        #self.D1.btn_D_end.setEnabled(False)
    def scan_pause(self):
        self.scan_paused = True
        #self.btn_scan_start.setEnabled(True)
        #self.btn_scan_end.setEnabled(False)
        #self.btn_scan_pause.setEnabled(False)
        self.mrc_mnc.oneD_scan_pause()
        self.trigger_oneD_scan_status.emit('oneD_scan_pause')
    def scan_resume(self):
        self.scan_paused = False
        #self.btn_scan_start.setEnabled(False)
        #self.btn_scan_end.setEnabled(True)
        #self.btn_scan_pause.setEnabled(True)
        self.mrc_mnc.oneD_scan_resume()
        self.trigger_oneD_scan_status.emit('oneD_scan_resume')

    def start_count(self,scantype):
        #print(scantype)
        if self.count_paused:
            self.resume_count()
        else:
            try:
               self.img_widget.clear_show()
            except:
               pass
            try:
               self.xbpm_widget.clear_show()
            except:
               pass
            #self.img_widget.clear_show()
            #self.xbpm_widget.clear_show()
            D_time = self.D1.detector_now_lineedit.text()
            D_name = self.D1.detector_combox.currentText()
            D_loop = self.D1.loop_lineedit.text()
            self.mrc_mnc.D_start(D_name, D_time, D_loop, str(scantype))
    def end_count(self):
        self.mrc_mnc.D_end()
        D_name = self.D1.detector_combox.currentText()
        print(D_name)
        if D_name in self.detector_image:
            self.mrc_mnc.D_image_cam(D_name)
        else:
            pass
        #self.D1.btn_D_start.setEnabled(True)
        #self.D1.btn_D_pause.setEnabled(False)
        #self.D1.btn_D_end.setEnabled(False)
        self.btn_scan_start.setEnabled(True)
        #self.btn_scan_end.setEnabled(False)
        #self.btn_scan_pause.setEnabled(False)
    def pause_count(self):
        self.count_paused = True
        self.mrc_mnc.D_pause()
        #self.D1.btn_D_start.setEnabled(True)
        #self.D1.btn_D_pause.setEnabled(False)
        #self.D1.btn_D_end.setEnabled(False)

    def resume_count(self):
        self.count_paused = False
        self.mrc_mnc.D_resume()
        #self.D1.btn_D_start.setEnabled(True)
        #self.D1.btn_D_pause.setEnabled(False)
        #self.D1.btn_D_end.setEnabled(False)

 
if __name__ == '__main__':        
    app = QApplication(sys.argv)
    w = one_image_scan()
    w.show()
    sys.exit(app.exec_())
